@@ -47,7 +47,8 @@
 #include "appl.h"
 #include "debug.h"
 #include "evnthndl.h"
-#include "memdefs.h"
+#include "gemdefs.h"
+#include "mintdefs.h"
 #include "menu.h"
 #include "mesagdef.h"
 #include "misc.h"
@@ -61,6 +62,7 @@
  ****************************************************************************/
 
 #define MAX_NUM_APPS 32
+
 #define AES_MESSAGES 12
 #define AES_OBJECTS  13
 #define AES_FORM     14
@@ -97,6 +99,39 @@ static AP_LIST *search_mpid(WORD pid)
 	return al;	
 }
 
+
+static WORD do_appl_read(WORD apid,WORD msgpipe,WORD length,void *m) {
+	
+	if((apid == APR_NOWAIT) && (Finstat(msgpipe) < length)) {
+		return 0;
+	};
+	
+	if(Fread(msgpipe,length,m) < 0) {
+		return 0;
+	};
+		
+	return 1;
+}
+
+void CDECL handle_signal(LONG signal) {
+	LONG ret;
+	
+	switch(signal) {
+	case SIGCHLD:
+/*
+		ret = Pwait();
+
+		DB_printf("Child died. Pid %d. Exit code %d (0x%x)",(WORD)(ret >> 16),(WORD)ret,(WORD)ret);
+*/
+		break;
+	default:
+		;
+/*
+		DB_printf("Unknown signal %ld",signal);
+*/
+	}
+}
+
 /****************************************************************************
  * Public functions                                                         *
  ****************************************************************************/
@@ -115,7 +150,7 @@ Appl_init_module(void) /*                                                   */
 		apps[i].id = -1;
 		apps[i].eventpipe = -1;
 		apps[i].msgpipe = -1;
-		apps[i].rsc = NULL;
+		apps[i].rshdr = NULL;
 		apps[i].deskbg = NULL;
 		apps[i].menu = NULL;
 		apps[i].deskmenu = -1;
@@ -147,58 +182,14 @@ WORD               /* Id of topped application.                             */
 Appl_get_top(void) /*                                                       */
 /****************************************************************************/
 {
-	return globals.ap_pri->ai->id;
+	if(globals.ap_pri) {
+		return globals.ap_pri->ai->id;
+	}
+	else {
+		return -1;
+	};
 }
 
-/****************************************************************************
- * Appl_get_vid                                                             *
- *  Get virtual vdi workstation of an application.                          *
- ****************************************************************************/
-WORD              /* Workstation handle or -1.                              */
-Appl_get_vid(     /*                                                        */
-WORD apid)        /* Id of application.                                     */
-/****************************************************************************/
-{
-	return apps[apid].vid;
-}
-
-/****************************************************************************
- * Appl_get_eventpipe                                                       *
- *  Get eventpipe of an application.                                        *
- ****************************************************************************/
-WORD                /* Eventpipe handle or -1.                              */
-Appl_get_eventpipe( /*                                                      */
-WORD apid)          /* Id of application.                                   */
-/****************************************************************************/
-{
-	return apps[apid].eventpipe;
-}
-
-/****************************************************************************
- * Appl_get_msgpipe                                                         *
- *  Get message pipe of an application.                                     *
- ****************************************************************************/
-WORD                /* Message pipe handle or -1.                           */
-Appl_get_msgpipe(   /*                                                      */
-WORD apid)          /* Id of application.                                   */
-/****************************************************************************/
-{
-	return apps[apid].msgpipe;
-}
-
-/****************************************************************************
- * Appl_get_rsrc                                                            *
- *  Get the resource tree of an application                                 *
- ****************************************************************************/
-RSHDR *           /* Resource tree, or NULL.                                */
-Appl_get_rsrc(    /*                                                        */
-WORD apid)        /* Id of application.                                     */
-/****************************************************************************/
-{
-	return apps[apid].rsc;
-}
-
-
 /****************************************************************************
  * Appl_search_apid                                                         *
  *  Find AES-id & return AP_LIST entry for that.                            *
@@ -221,54 +212,24 @@ WORD apid)        /* Application id of the searched application.            */
 	return al;	
 }
 
-/****************************************************************************
- * Appl_set_rsrc                                                            *
- *  Set the resource tree of an application                                 *
- ****************************************************************************/
-WORD              /* 0 if ok or -1.                                         */
-Appl_set_rsrc(    /*                                                        */
-WORD apid,        /* Id of application.                                     */
-RSHDR *tree)      /* Resource tree.                                         */
-/****************************************************************************/
-{
-	if(apps[apid].id != -1) {
-		apps[apid].rsc = tree;
-		return 0;
-	};
-	
-	return -1;
-}
-
 /****************************************************************************
  * Appl_get_deskbg                                                          *
- *  Get the resource tree of the desktop of an application                  *
+ *  Get the resource tree of the top desktop.                               *
  ****************************************************************************/
-OBJECT *          /* Resource tree, or NULL.                                */
-Appl_get_deskbg(  /*                                                        */
-WORD apid)        /* Id of application.                                     */
+OBJECT *               /* Resource tree, or NULL.                           */
+Appl_get_deskbg(void)  /*                                                   */
 /****************************************************************************/
 {
-	if(apid == TOP_APPL) {
-		if(globals.ap_pri) {
-			return globals.ap_pri->ai->deskbg;
-		};
+	OBJECT  *retval = NULL;
+	AP_INFO *ai;
 		
-		return NULL;
-	}
-	else if(apid == DESK_OWNER) {
-		OBJECT  *retval = NULL;
-		AP_INFO *ai;
+	ai = Appl_internal_info(DESK_OWNER);
 		
-		ai = Appl_internal_info(DESK_OWNER);
-		
-		if(ai) {
-			retval = ai->deskbg;
-		};
-		
-		return retval;
+	if(ai) {
+		retval = ai->deskbg;
 	};
-	
-	return apps[apid].deskbg;
+		
+	return retval;
 }
 
 /****************************************************************************
@@ -278,6 +239,7 @@ WORD apid)        /* Id of application.                                     */
 WORD              /* 0 if ok or -1.                                         */
 Appl_set_deskbg(  /*                                                        */
 WORD apid,        /* Id of application.                                     */
+WORD vid,         /* VDI workstation id.                                    */
 OBJECT *tree)     /* Resource tree.                                         */
 /****************************************************************************/
 {
@@ -287,11 +249,11 @@ OBJECT *tree)     /* Resource tree.                                         */
 		apps[apid].deskbg = tree;
 
 		Rdwrs_operation(ASTARTREAD);
-		deskbg = Appl_get_deskbg(DESK_OWNER);
+		deskbg = Appl_get_deskbg();
 		Rdwrs_operation(AENDREAD);
 	
 		if(((deskbg == tree) && (deskbg != olddeskbg)) || !tree) {
-			Wind_update_deskbg(Appl_get_vid(apid));
+			Wind_update_deskbg(vid);
 		};
 	
 		return 0;
@@ -374,7 +336,8 @@ WORD newmsg)      /* Messages understood.                                   */
  ****************************************************************************/
 WORD              /* Previously topped application.                         */
 Appl_top(         /*                                                        */
-WORD apid)        /* Id of application.                                     */
+WORD apid,        /* Id of application.                                     */
+WORD vid)         /* VDI workstation id.                                    */
 /****************************************************************************/
 {
 	AP_LIST **al;
@@ -411,8 +374,6 @@ WORD apid)        /* Id of application.                                     */
 	Rdwrs_quick(AENDWRITE);
 	
 	if(deskbg && deskbgcount) {
-		WORD vid = Appl_get_vid(apid);
-
 		Rdwrs_operation(WSTARTREAD);
 		Wind_update_deskbg(vid);
 		Rdwrs_operation(WENDREAD);
@@ -556,15 +517,15 @@ WORD type)          /* Type of application (appl, acc, desktop etc)         */
 	al->ai->type = type;
 	al->ai->newmsg = 0;
 	
-	sprintf(s,"u:\\pipe\\applmsg.%03x",al->ai->id);		
+	sprintf(s,"u:\\pipe\\applmsg.%03x",al->ai->pid);		
 	al->ai->msgname = (BYTE *)Mxalloc(strlen(s) + 1,GLOBALMEM);
 	strcpy(al->ai->msgname,s);
 
-	sprintf(s,"u:\\pipe\\applevnt.%03x",al->ai->id);		
+	sprintf(s,"u:\\pipe\\applevnt.%03x",al->ai->pid);		
 	al->ai->eventname = (BYTE *)Mxalloc(strlen(s) + 1,GLOBALMEM);
 	strcpy(al->ai->eventname,s);
 		
-	al->ai->rsc = 0L;
+	al->ai->rshdr = 0L;
 	
 	/* open up a vdi workstation to use in the application */
 
@@ -608,8 +569,9 @@ void	apinfofree(WORD id) {
 			Fclose(altemp->ai->eventpipe);
 			altemp->ai->eventpipe = -1;
 			
-			if(altemp->ai->rsc)
-				Mfree(altemp->ai->rsc);	/*free resource memory*/
+			if(altemp->ai->rshdr) {
+				Mfree(altemp->ai->rshdr);	/*free resource memory*/
+			};
 
 			Mfree(altemp->ai->msgname);
 			Mfree(altemp->ai->eventname);
@@ -624,17 +586,31 @@ void	apinfofree(WORD id) {
 
 /* 0x000a appl_init */
 
-void	Appl_init(AES_PB *apb) {
+/****************************************************************************
+ * Appl_do_init                                                             *
+ *  Implementation of appl_init().                                          *
+ ****************************************************************************/
+WORD                   /* Application id, or -1.                            */
+Appl_do_init(          /*                                                   */
+GLOBAL_ARRAY *global)  /* Global array.                                     */
+/****************************************************************************/
+{
 	AP_INFO	*ai;
 	AP_LIST	*al;
 	WORD    pid = Pgetpid();
-	
+	WORD    apid;
+
+	Psignal(SIGCHLD,(LONG)handle_signal);
+	Psignal(SIGQUIT,(LONG)handle_signal);
+	Psignal(SIGBUS,(LONG)handle_signal);
+	Psignal(SIGSEGV,(LONG)handle_signal);
+
 	Rdwrs_operation(ASTARTWRITE);
 
 	al = search_mpid(pid);
 
 	if(!al) {			/* A process is only allowed to have one apid */
-		/* Has a info structure already been reserved? */
+		/* Has an info structure already been reserved? */
 		
 		AP_LIST **awalk = &ap_resvd;
 		
@@ -673,59 +649,63 @@ void	Appl_init(AES_PB *apb) {
 	};
 
 	if(ai) {
-		apb->int_out[0] = ai->id;	
-		apb->global[0] = 0x0410;
-		apb->global[1] = 255;
-		apb->global[2] = ai->id;
+		apid = ai->id;	
+
+		global->version = 0x0410;
+		global->numapps = -1;
+		global->apid = ai->id;
+		global->appglobal = 0L;
+		global->rscfile = 0L;
+		global->rshdr = 0L;
+		global->resvd1 = 0;
+		global->resvd2 = 0;
+		global->int_info = ai;
+		global->maxchar = 0;
+		global->minchar = 0;
 	}
 	else {
-		apb->int_out[0] = -1;
+		apid = -1;
 	};
 	
 	Rdwrs_operation(AENDWRITE);
 	
-	if(apb->int_out[0] >= 0) {
+	if(apid >= 0) {
 		BYTE fname[128],cmdlin[128],menuentry[21],*tmp;
 		
 		Misc_get_loadinfo(128,cmdlin,fname);
 
 		strcpy(menuentry,"  ");
-		if((tmp = strrchr(fname,'\\'))) {
+		
+		tmp = strrchr(fname,'\\');
+		
+		if(tmp) {
 			tmp++;
 		}
 		else {
 			tmp = fname;
 		};
+		
 		strncat(menuentry,tmp,20);
 	 
 	 	if(ai->type & APP_APPLICATION) {
 			Menu_do_register(ai->id, menuentry);
 		};
 	};
+	
+	return apid;
+}
+
+void	Appl_init(AES_PB *apb) {
+	apb->int_out[0] = Appl_do_init(apb->global);
 }
 
 /* 0x000b appl_read */
 
-WORD do_appl_read(WORD apid,WORD length,void *m) {
-	WORD	pnr;
-	
-	pnr = Appl_get_msgpipe(apid);
-	
-	if(pnr < 0)
-		return 0;
-
-	if((apid == APR_NOWAIT) && (Finstat(pnr) < length))
-		return 0;
-	
-	if(Fread(pnr,length,m) < 0)
-		return 0;
-		
-	return 1;
-}
-
 void	Appl_read(AES_PB *apb) {
-	apb->int_out[0] = do_appl_read(apb->int_in[0],apb->int_in[1],
-							(void *)apb->addr_in[0]);
+	apb->int_out[0] = do_appl_read(apb->int_in[0],
+																apb->global->int_info->msgpipe,
+																apb->int_in[1],
+																(void *)apb->addr_in[0]);
 }
 
 /* 0x000c appl_write */
@@ -772,7 +752,7 @@ void *m)       /* Pointer to message structure.                             */
 	Rdwrs_quick(AENDREAD);	
 
 	return 1;
-};
+}
 
 void	Appl_write(AES_PB *apb) {
 	apb->int_out[0] =	
@@ -796,12 +776,14 @@ BYTE *fname)      /* File name of application to seek.                      */
 	BYTE    pname[ 30];
 
 	w = (LONG)fname & 0xffff0000l;
-	if ( w == 0xffff0000l)
-	{
+	if(w == 0xffff0000l) {
 	/* convert MiNT to AES pid */
 		al = search_mpid((WORD)((LONG)fname));
-		if ( al)
+		
+		if(al) {
 			return al->ai->id;
+		};
+		
 		return -1;
 	}
 	
@@ -820,16 +802,20 @@ BYTE *fname)      /* File name of application to seek.                      */
 	Fsetdta( &newdta);
 	w = -1;
 	sprintf(pname, "u:\\proc\\%s.*", fname);
-	if(Fsfirst(pname, 0) == 0) 
+
+	if(Fsfirst(pname, 0) == 0) {
 		w = atoi(&newdta.dta_name[9]);
+	};
 
 	Fsetdta(olddta);
 
 /* map the MINT-pid to aes */
-	if (w != -1) {
-		al = search_mpid( w);
-		if(al)
+	if(w != -1) {
+		al = search_mpid((WORD)w);
+
+		if(al) {
 			return al->ai->id;
+		};
 	};
 	
 	return -1;
@@ -845,7 +831,7 @@ AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
 	if ( apb->addr_in[0] == 0)	/* return our own pid */
-		apb->int_out[0] = apb->global[ 2];
+		apb->int_out[0] = apb->global->apid;
 	else
 		apb->int_out[0] = Appl_do_find((BYTE *)apb->addr_in[0]);
 }
@@ -917,7 +903,7 @@ AES_PB *apb)      /* AES parameter block.                                   */
 {
 	Rdwrs_operation(ASTARTREAD);
 	
-	apb->int_out[ 0] = Appl_do_search(apb->global[2], apb->int_in[0], 
+	apb->int_out[ 0] = Appl_do_search(apb->global->apid, apb->int_in[0], 
 	     (BYTE *)apb->addr_in[0],
 	     &apb->int_out[1],
 	     &apb->int_out[2]);
@@ -927,20 +913,44 @@ AES_PB *apb)      /* AES parameter block.                                   */
 
 /* 0x0013 appl_exit */
 
-void	Appl_exit(AES_PB *apb) {
+/****************************************************************************
+ * Appl_do_exit                                                             *
+ *  Implementation of appl_exit().                                          *
+ ****************************************************************************/
+WORD           /* 0 if error, or 1.                                         */
+Appl_do_exit(  /*                                                           */
+WORD apid,     /* Application id.                                           */
+WORD vid)      /* VDI workstation id.                                       */
+/****************************************************************************/
+{
 	/*clean up*/
+
+	Menu_bar_remove(apid);
+
+	Menu_unregister(apid);
 	
-	Menu_bar_remove(apb->global[2]);
-	Menu_unregister(apb->global[2]);
-	Appl_set_deskbg(Appl_get_vid(apb->global[2]),NULL);
-	
+	Appl_set_deskbg(apid,vid,NULL);
+
+	Wind_do_new(vid,apid);
+
 	Rdwrs_operation(ASTARTWRITE);
 
-	apinfofree(apb->global[2]);
-	
-	apb->int_out[0] = 1;
-	
+	apinfofree(apid);
+		
 	Rdwrs_operation(AENDWRITE);
+	
+	Rdwrs_operation(MSTARTWRITE);
+	Rdwrs_operation(ASTARTREAD);
+	Menu_update_appl();
+
+	Rdwrs_operation(AENDREAD);
+	Rdwrs_operation(MENDWRITE);
+
+	return 1;
+}
+
+void	Appl_exit(AES_PB *apb) {
+	apb->int_out[0] = Appl_do_exit(apb->global->apid,apb->global->int_info->vid);
 }
 
 /****************************************************************************
