@@ -16,7 +16,7 @@
    Added standard header.
    Mouse_gain_mctrl() and Mouse_release_mctrl() added.
 
-  960129
+  960129 cg
    Changed name form mouse.c to evnthndl.c.
    Public routines changed prefix from Mouse_ to Evhd_.
    
@@ -24,6 +24,10 @@
    mouse arrow changement while window movement, sizing and sliding
    realtime slider.
 
+  960623 cg
+   Fixed mover grabbing bug; if the mouse was moved during click on
+   window mover the window was topped / bottomed instead of dragged.
+   
  Copyright notice
   The copyright to the program code herein belongs to the authors. It may
   be freely duplicated and distributed without fee, but not charged for.
@@ -48,14 +52,17 @@
 #include "debug.h"
 #include "evnt.h"
 #include "evnthndl.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "graf.h"
-#include "memdefs.h"
+#include "mintdefs.h"
 #include "mesagdef.h"
 #include "misc.h"
 #include "mousedev.h"
 #include "objc.h"
 #include "resource.h"
+#include "rdwrs.h"
+#include "rsrc.h"
 #include "types.h"
 #include "vdi.h"
 #include "wind.h"
@@ -66,9 +73,6 @@
  * Macros                                                                   *
  ****************************************************************************/
 
-#define MCTRL_SEM 0x4d435482L /*"MCTR"*/
-#define	MOUSESEM	0x4d4f4556L	/*MOEV*/
-
 #define EVHD_APID     (-2)
 #define EVHD_WAITTIME 200
 
@@ -104,8 +108,9 @@ static struct {
 };
 
 static CSEMA mctrl = {
-	MCTRL_SEM,
 	-1L,
+	-1,
+	-1,
 	0,
 	-1
 };
@@ -116,6 +121,8 @@ static WORD mouse_button;
 
 static WORD iconify_x = 0;
 
+static LONG mouse_sem;
+
 /****************************************************************************
  * Local functions (use static!)                                            *
  ****************************************************************************/
@@ -158,7 +165,7 @@ static LONG	get_evntpacket(EVNTREC *er,WORD maxwait) {
 	if(Fselect(maxwait,&rhndl,NULL,0L) == 0) {
 		return 0;
 	};
-		
+	
 	if(rhndl & stdinrfds) {
 		ULONG c = Fgetchar(STDIN_FILENO,0);
 		
@@ -167,9 +174,8 @@ static LONG	get_evntpacket(EVNTREC *er,WORD maxwait) {
 		evntglbl.last_evntrec.ap_event = APPEVNT_KEYBOARD;
 		evntglbl.last_evntrec.ap_value = ((c & 0xff) << 16) +
 			((c & 0x00ff0000L) << 8) + kb_shift;
-
-	};
-
+	}
+	
 	if(rhndl & mrfds)  {
  		Fread(evntglbl.mousefd, sizeof(EVNTREC), 
  		       &evntglbl.last_evntrec);
@@ -216,7 +222,7 @@ static void handle_arrow_click(WORD win_id,WORD elem,WORD msg) {
 	WINSTRUCT *win;
 	WORD      owner;
 	
-	Evhd_gain_mctrl(EVHD_APID,MCTRL_BUTTON);
+	Evhd_gain_mctrl(EVHD_APID,FALSE);
 
 	mesag.type = WM_ARROWED;
 	mesag.sid = 0;
@@ -260,7 +266,7 @@ static void handle_arrow_click(WORD win_id,WORD elem,WORD msg) {
 
 	Appl_do_write(owner,16,&mesag);
 
-	Evhd_release_mctrl(EVHD_APID);		
+	Evhd_release_mctrl();
 }
 
 static void handle_mover_click(WORD win_id) {
@@ -270,7 +276,6 @@ static void handle_mover_click(WORD win_id) {
 	WINSTRUCT *win;
 	EVNTREC   er;
 	COMMSG    mesag;
-	WORD      xyarray[10];
   
 	Rdwrs_operation(WSTARTWRITE);
  
@@ -293,6 +298,7 @@ static void handle_mover_click(WORD win_id) {
     };
     
     if(timeleft < 0) {
+    	timeleft = 0;
       break;
     };
     
@@ -392,76 +398,51 @@ static void handle_mover_click(WORD win_id) {
                !(er.ap_value & LEFT_BUTTON)));
     }
     else {
-			Wind_beg_update(-3);
+    	RECT bound;
+			RECT winsize;
+			WORD owner;
+    	
+    	bound.y = globals.clheight + 3;
+    	bound.height = 30000;
+    	bound.x = -15000;
+    	bound.width = 30000;
+    	
+			Wind_beg_update();
 
 			Rdwrs_operation(WSTARTREAD);
 
       win = Wind_find_description(win_id);
 
-      Vdi_vsl_type(evntglbl.evid,3);
-      Vdi_vswr_mode(evntglbl.evid,MD_XOR);
-      
-      xyarray[0] = win->totsize.x;
-      xyarray[1] = win->totsize.y;
-      xyarray[2] = win->totsize.x
-        + win->totsize.width - 1;
-      xyarray[3] = xyarray[1];
-      xyarray[4] = xyarray[2];
-      xyarray[5] = win->totsize.y
-        + win->totsize.height - 1;
-      xyarray[6] = xyarray[0];
-      xyarray[7] = xyarray[5];
-      xyarray[8] = xyarray[0];
-      xyarray[9] = xyarray[1];
+			winsize = win->totsize;
+			owner = win->owner;
+
+			Rdwrs_operation(WENDREAD);
+
+			globals.mouse_button = mouse_button;
+
+			Graf_do_dragbox(evntglbl.evid,evntglbl.mousefd,
+					winsize.width,winsize.height,
+					winsize.x,winsize.y,
+					&bound,
+					&mesag.msg1,&mesag.msg2);
+				          
+		  Vdi_vq_mouse(globals.vid,&mouse_button,&mouse_x,&mouse_y);
 
-      do {
-        xyarray[0] += mouse_x - last_x;
-        xyarray[1] += mouse_y - last_y;
-        xyarray[2] += mouse_x - last_x;
-        xyarray[3] = xyarray[1];
-        xyarray[4] = xyarray[2];
-        xyarray[5] += mouse_y - last_y;
-        xyarray[6] = xyarray[0];
-        xyarray[7] = xyarray[5];
-        xyarray[8] = xyarray[0];
-        xyarray[9] = xyarray[1];
-        
-        last_x = mouse_x;
-        last_y = mouse_y;
-        
-        Vdi_v_hide_c(evntglbl.evid);
-        Vdi_v_pline(evntglbl.evid,5,xyarray);
-        Vdi_v_show_c(evntglbl.evid,1);
-        
-        get_evntpacket(&er,0);
-        update_local_mousevalues(&er);
-        
-        Vdi_v_hide_c(evntglbl.evid);
-        Vdi_v_pline(evntglbl.evid,5,xyarray);
-        Vdi_v_show_c(evntglbl.evid,1);
-      }while(!((er.ap_event == APPEVNT_BUTTON) &&
-               !(er.ap_value & LEFT_BUTTON)));
-      
-      
       mesag.type = WM_MOVED;
       mesag.sid = 0;
       mesag.length = 0;
       mesag.msg0 = win_id;
-      mesag.msg1 = xyarray[0];
-      mesag.msg2 = xyarray[1];
 
       if(mesag.msg2 < (globals.clheight + 3)) {
        	mesag.msg2 = globals.clheight + 3;
       }
 
-      mesag.msg3 = win->totsize.width;
-      mesag.msg4 = win->totsize.height;
+      mesag.msg3 = winsize.width;
+      mesag.msg4 = winsize.height;
       
-      Appl_do_write(win->owner,16,&mesag);
-
-			Rdwrs_operation(WENDREAD);
+      Appl_do_write(owner,MSG_LENGTH,&mesag);
 			
-      Wind_end_update(-3);
+      Wind_end_update();
     };
     
    	Graf_do_mouse(evntglbl.evid,M_RESTORE,NULL);
@@ -583,7 +564,7 @@ static void handle_slider_click(WORD win_id, WORD elem) {
         }
     }
     else {
-        Wind_beg_update(-3);
+        Wind_beg_update();
         Rdwrs_operation(WSTARTREAD);
 
         win = Wind_find_description(win_id);
@@ -667,7 +648,7 @@ static void handle_slider_click(WORD win_id, WORD elem) {
         Appl_do_write(win->owner,16,&mesag);
 
         Rdwrs_operation(WENDREAD);
-        Wind_end_update(-3);
+        Wind_end_update();
     }
 
     Rdwrs_operation(WSTARTREAD);
@@ -679,190 +660,6 @@ static void handle_slider_click(WORD win_id, WORD elem) {
     Graf_do_mouse(evntglbl.evid,M_RESTORE, NULL);  
 }
 
-/*
-static void handle_slider_click(WORD win_id,WORD elem) {
-  WORD      last_x = mouse_x;
-  WORD      last_y = mouse_y;
-  WORD      new_x = last_x,
-            new_y = last_y,
-            waittime = EVHD_WAITTIME;
-  WORD      dx,dy;
-	WINSTRUCT *win;
-	EVNTREC   er;
-	COMMSG    mesag;
-	WORD      xyarray[10];
-	RECT      elemrect,bgrect;
-	WORD      bg;
-  
-  if(elem == WVSLIDER) {
-  	bg = WVSB;
-  }
-  else {
-  	bg = WHSB;
-  };
-  
-	Rdwrs_operation(WSTARTWRITE);
- 
-  win = Wind_find_description(win_id);
-  
-  win->tree[elem].ob_state |= SELECTED;
-  
-  Wind_draw_elements(evntglbl.evid,win,&globals.screen,elem);
- 
- 	Objc_do_offset(win->tree,elem,(WORD *)&elemrect);
- 	elemrect.width = win->tree[elem].ob_width;
- 	elemrect.height = win->tree[elem].ob_height;
-  
-  dx = last_x - elemrect.x;
-  dy = last_y - elemrect.y;
-  
- 	Objc_do_offset(win->tree,bg,(WORD *)&bgrect);
- 	bgrect.width = win->tree[bg].ob_width;
- 	bgrect.height = win->tree[bg].ob_height;
-  
-  Rdwrs_operation(WENDWRITE);
-  
-	Wind_beg_update(-3);
-
-	Rdwrs_operation(WSTARTREAD);
-
-  win = Wind_find_description(win_id);
-
-  Vdi_vsl_type(evntglbl.evid,3);
-  Vdi_vswr_mode(evntglbl.evid,MD_XOR);
-  
-  xyarray[0] = elemrect.x;
-  xyarray[1] = elemrect.y;
-  xyarray[2] = elemrect.x
-				       + elemrect.width - 1;
-  xyarray[3] = xyarray[1];
-  xyarray[4] = xyarray[2];
-  xyarray[5] = elemrect.y
-				       + elemrect.height - 1;
-  xyarray[6] = xyarray[0];
-  xyarray[7] = xyarray[5];
-  xyarray[8] = xyarray[0];
-  xyarray[9] = xyarray[1];
-
-  Vdi_v_hide_c(evntglbl.evid);
-  Vdi_v_pline(evntglbl.evid,5,xyarray);
-  Vdi_v_show_c(evntglbl.evid,1);
-      
-  while(1) {
-  	WORD new_x,new_y;
-  	
-    get_evntpacket(&er,0);
-    update_local_mousevalues(&er);
-    
-    if((er.ap_event == APPEVNT_BUTTON) &&
-           !(er.ap_value & LEFT_BUTTON)) {
-    	break;
-    };  
-  	
-  	if(elem == WHSLIDER) {
-  		new_y = last_y;
-  		
-  		if((mouse_x - dx + elemrect.width) > (bgrect.x + bgrect.width)) {
-  			new_x = bgrect.x + bgrect.width - elemrect.width + dx;
-  		}
-  		else if((mouse_x - dx) < bgrect.x) {
-  			new_x = bgrect.x + dx;
-  		}
-  		else {
-  			new_x = mouse_x;
-  		};
-
-			if(new_x != last_x) {
-	      Vdi_v_hide_c(evntglbl.evid);
-	      Vdi_v_pline(evntglbl.evid,5,xyarray);
-	      Vdi_v_show_c(evntglbl.evid,1);
-      
-        xyarray[0] += new_x - last_x;
-        xyarray[2] += new_x - last_x;
-	      xyarray[4] = xyarray[2];
-  	    xyarray[6] = xyarray[0];
-    	  xyarray[8] = xyarray[0];
-
-	      Vdi_v_hide_c(evntglbl.evid);
-	      Vdi_v_pline(evntglbl.evid,5,xyarray);
-	      Vdi_v_show_c(evntglbl.evid,1);
-    	};
-  	}
-  	else {
-  		new_x = last_x;
-  		
-  		if((mouse_y - dy + elemrect.height) > (bgrect.y + bgrect.height)) {
-  			new_y = bgrect.y + bgrect.height - elemrect.height + dy;
-  		}
-  		else if((mouse_y - dy) < bgrect.y) {
-  			new_y = bgrect.y + dy;
-  		}
-  		else {
-  			new_y = mouse_y;
-  		};
-
-  		if(new_y != last_y) {
-        Vdi_v_hide_c(evntglbl.evid);
-		    Vdi_v_pline(evntglbl.evid,5,xyarray);
-        Vdi_v_show_c(evntglbl.evid,1);
-      
-      	xyarray[1] += new_y - last_y;
-        xyarray[3] = xyarray[1];
-	      xyarray[5] += new_y - last_y;
-  	    xyarray[7] = xyarray[5];
-    	  xyarray[9] = xyarray[1];
-
-        Vdi_v_hide_c(evntglbl.evid);
-		    Vdi_v_pline(evntglbl.evid,5,xyarray);
-    		Vdi_v_show_c(evntglbl.evid,1);
-      };
-  	};
-    
-    last_x = new_x;
-    last_y = new_y;
-  };
-  
-  Vdi_v_hide_c(evntglbl.evid);
-  Vdi_v_pline(evntglbl.evid,5,xyarray);
-  Vdi_v_show_c(evntglbl.evid,1);
-      
-	if(elem == WHSLIDER) {
-    mesag.type = WM_HSLID;
-    if(bgrect.width == elemrect.width) {
-    	mesag.msg1 = 0;
-    }
-    else {
-			mesag.msg1 = (WORD)((((LONG)last_x - (LONG)dx - (LONG)bgrect.x) * (LONG)1000L) / 
-										((LONG)bgrect.width - (LONG)elemrect.width));
-		};
-	}
-	else {
-    mesag.type = WM_VSLID;
-    if(bgrect.height == elemrect.height) {
-    	mesag.msg1 = 0;
-    }
-    else {
-			mesag.msg1 = (WORD)((((LONG)last_y - (LONG)dy - (LONG)bgrect.y) * (LONG)1000L) / 
-										((LONG)bgrect.height - (LONG)elemrect.height));
-		};
-	};
-	      
-  mesag.sid = 0;
-  mesag.length = 0;
-  mesag.msg0 = win_id;
-  
-  Appl_do_write(win->owner,16,&mesag);
-
-  win = Wind_find_description(win_id);
-  
-  win->tree[elem].ob_state &= ~SELECTED;
-  Wind_draw_elements(evntglbl.evid,win,&globals.screen,elem);
-  
-	Rdwrs_operation(WENDREAD);
-
-  Wind_end_update(-3);
-}
-*/
 
 static void	handle_button(WORD newbutton) {
   EVNTREC	er;
@@ -918,7 +715,6 @@ static void	handle_button(WORD newbutton) {
       	WORD	obj = Objc_do_find(win->tree,0,9,mouse_x,mouse_y,0);
       	WORD	owner = win->owner;
 				COMMSG	mesag;				
-				WORD	xyarray[10];
       
 				Rdwrs_operation(WENDREAD);
       	
@@ -1109,54 +905,29 @@ static void	handle_button(WORD newbutton) {
 				             !(er.ap_value & LEFT_BUTTON)));
 				 	}
 					else {
-					  Vdi_vsl_type(evntglbl.evid,3);
-					  Vdi_vswr_mode(evntglbl.evid,MD_XOR);
-					  
-					  xyarray[0] = win->totsize.x;
-					  xyarray[1] = win->totsize.y;
-					  xyarray[3] = xyarray[1];
-					  xyarray[6] = xyarray[0];
-					  xyarray[8] = xyarray[0];
-					  xyarray[9] = xyarray[1];
-				
-						Wind_beg_update(-3);
-					  
-					  do {
-					    xyarray[2] = mouse_x;
-					    xyarray[4] = xyarray[2];
-					    xyarray[5] = mouse_y;
-					    xyarray[7] = xyarray[5];
-					    
-					    Vdi_v_hide_c(evntglbl.evid);
-					    Vdi_v_pline(evntglbl.evid,5,xyarray);
-	            Vdi_v_show_c(evntglbl.evid,1);
-	            
-	            get_evntpacket(&er,0);
-	            update_local_mousevalues(&er);
-	            
-	            Vdi_v_hide_c(evntglbl.evid);
-	            Vdi_v_pline(evntglbl.evid,5,xyarray);
-	            Vdi_v_show_c(evntglbl.evid,1);
-	          }while(!((er.ap_event == APPEVNT_BUTTON) &&
-	                   !(er.ap_value & LEFT_BUTTON)));
-	          
-	          Wind_end_update(-3);
-	          
+						globals.mouse_button = mouse_button;
+
+						Graf_do_rubberbox(evntglbl.evid,evntglbl.mousefd,
+								win->totsize.x,win->totsize.y,100,100,
+								&mesag.msg3,&mesag.msg4);
+							          
+					  Vdi_vq_mouse(globals.vid,&mouse_button,&mouse_x,&mouse_y);
+
 	          mesag.type = WM_SIZED;
 	          mesag.sid = 0;
 	          mesag.length = 0;
 	          mesag.msg0 = win_id;
 	          mesag.msg1 = win->totsize.x;
 	          mesag.msg2 = win->totsize.y;
-	          mesag.msg3 = mouse_x - win->totsize.x - 1;
-	          mesag.msg4 = mouse_y - win->totsize.y - 1;
+	          mouse_x = mesag.msg3 + win->totsize.x - 1;
+	          mouse_y = mesag.msg4 + win->totsize.y - 1;
 
 						Rdwrs_operation(WENDWRITE);
 
 	          Appl_do_write(owner,16,&mesag);
 	        };
 
-					Wind_beg_update(-3);
+					Wind_beg_update();
 					
 					Rdwrs_operation(WSTARTREAD);
 	        
@@ -1167,7 +938,7 @@ static void	handle_button(WORD newbutton) {
 
 					Rdwrs_operation(WENDREAD);
 					
-					Wind_end_update(-3);
+					Wind_end_update();
 	        break;
 
 				case WAPP:	        
@@ -1198,7 +969,9 @@ static void	handle_button(WORD newbutton) {
 	      		
 						Rdwrs_operation(WSTARTREAD);
 						
-						if((win = Wind_find_description(win_id))) {
+						win = Wind_find_description(win_id);
+						
+						if(win) {
 			      	Objc_do_offset(win->tree,WHSLIDER,xy);
 			      };
 		      	
@@ -1220,7 +993,9 @@ static void	handle_button(WORD newbutton) {
 	      		
 						Rdwrs_operation(WSTARTREAD);
 						
-						if((win = Wind_find_description(win_id))) {
+						win = Wind_find_description(win_id);
+						
+						if(win) {
 			      	Objc_do_offset(win->tree,WVSLIDER,xy);
 			      };
 		      	
@@ -1337,7 +1112,7 @@ static void check_rectevents(WORD x,WORD y) {
     return;                 /* the list */
   };
     
-  Psemaphore(2,MOUSESEM,-1);
+  Psemaphore(2,mouse_sem,-1);
   
   rl = &evntglbl.eventlist;
   
@@ -1411,7 +1186,7 @@ static void check_rectevents(WORD x,WORD y) {
     rl = &(*rl)->next;
   };
   
-  Psemaphore(3,MOUSESEM,-1);
+  Psemaphore(3,mouse_sem,-1);
 }
 
 static void	getmenuxpos(WORD *x,WORD *width) {
@@ -1423,7 +1198,9 @@ static void	getmenuxpos(WORD *x,WORD *width) {
 
 	Rdwrs_operation(ASTARTREAD);
 	
-	if((t = Appl_get_menu(TOP_MENU_OWNER))) {
+	t = Appl_get_menu(TOP_MENU_OWNER);
+	
+	if(t) {
 		start = t[t[0].ob_head].ob_head;
 	
 		*x = t[0].ob_x + t[t[0].ob_head].ob_x + t[start].ob_x;
@@ -1431,7 +1208,7 @@ static void	getmenuxpos(WORD *x,WORD *width) {
 	}
 	else {
 		DB_printf("%s: Line %d: getmenuxpos:\r\n"
-									"Couldn't find top menu.\r\n");
+									"Couldn't find top menu.\r\n",__FILE__,__LINE__);
 	};
 	
 	Rdwrs_quick(AENDREAD);
@@ -1479,8 +1256,8 @@ static WORD	handle_drop_down(WORD menubox,WORD title) {
 
 	OBJECT   *nmenu;
 	
-	if((deskbox == menubox) && (mouse_y >= pmenutad[0].ob_y)) {
-		nmenu = pmenutad;
+	if((deskbox == menubox) && (mouse_y >= globals.pmenutad[0].ob_y)) {
+		nmenu = globals.pmenutad;
 		entry = Objc_do_find(nmenu,0,9,mouse_x,mouse_y,0);
 	}
 	else {
@@ -1535,7 +1312,7 @@ static WORD	handle_drop_down(WORD menubox,WORD title) {
 		    	if(changebut & LEFT_BUTTON & (!mouse_button)) {
 						nmenu[entry].ob_state &= ~SELECTED;
 		
-						if(nmenu == pmenutad) {
+						if(nmenu == globals.pmenutad) {
 							AP_LIST *mr;
 							WORD    walk = entry - PMENU_FIRST;
 						
@@ -1645,16 +1422,16 @@ static WORD	handle_selected_title(void) {
 		if(box == deskbox) {
 			WORD i;
 			
-	    Objc_do_offset(menu.tree,box,&pmenutad[0].ob_x);
+	    Objc_do_offset(menu.tree,box,&globals.pmenutad[0].ob_x);
     
-	    pmenutad[0].ob_y += (menu.tree[menu.tree[box].ob_head].ob_height << 1);
-			pmenutad[0].ob_width = menu.tree[box].ob_width;
+	    globals.pmenutad[0].ob_y += (menu.tree[menu.tree[box].ob_head].ob_height << 1);
+			globals.pmenutad[0].ob_width = menu.tree[box].ob_width;
 
 			for(i = PMENU_FIRST; i <= PMENU_LAST; i++) {
-				pmenutad[i].ob_width = pmenutad[0].ob_width;
+				globals.pmenutad[i].ob_width = globals.pmenutad[0].ob_width;
 			}
 		
-			area.height = pmenutad[0].ob_height + (pmenutad[1].ob_height << 1) + 2;
+			area.height = globals.pmenutad[0].ob_height + (globals.pmenutad[1].ob_height << 1) + 2;
 		};
 
 		Rdwrs_operation(WSTARTWRITE);
@@ -1667,12 +1444,12 @@ static WORD	handle_selected_title(void) {
 		if(box == deskbox) {
 			RECT clip = area;
 			
-			clip.height = pmenutad[0].ob_y - area.y;
+			clip.height = globals.pmenutad[0].ob_y - area.y;
 			Objc_do_draw(evntglbl.evid,menu.tree,box,9,&clip);
 			
-			clip.y = pmenutad[0].ob_y;
-			clip.height = pmenutad[0].ob_height + 1;
-			Objc_do_draw(evntglbl.evid,pmenutad,0,9,&clip);
+			clip.y = globals.pmenutad[0].ob_y;
+			clip.height = globals.pmenutad[0].ob_height + 1;
+			Objc_do_draw(evntglbl.evid,globals.pmenutad,0,9,&clip);
 		}
 		else {
 			Objc_do_draw(evntglbl.evid,menu.tree,box,9,&area);
@@ -1695,9 +1472,9 @@ static WORD	handle_selected_title(void) {
 				if(!Misc_inside(&titlearea,mouse_x,mouse_y)) {
 	      	WORD closebox = 0;
 	      	
-	      	if((deskbox == box) && (mouse_y >= pmenutad[0].ob_y)) {
-	      		if(!Misc_inside((RECT *)&pmenutad[0].ob_x,mouse_x,mouse_y) ||
-	      			(Objc_do_find(pmenutad,0,9,mouse_x,mouse_y,0) < 0) ||
+	      	if((deskbox == box) && (mouse_y >= globals.pmenutad[0].ob_y)) {
+	      		if(!Misc_inside((RECT *)&globals.pmenutad[0].ob_x,mouse_x,mouse_y) ||
+	      			(Objc_do_find(globals.pmenutad,0,9,mouse_x,mouse_y,0) < 0) ||
 	      			handle_drop_down(box,title)) {
 	      			closebox = 1;
 	      		};
@@ -1777,24 +1554,23 @@ static void handle_menu(void) {
   };
 }
 
-static WORD mouseproc(LONG arg) {
+static WORD evnt_handler(LONG arg) {
   WORD	  work_in[] = {1,7,1,1,1,1,1,1,1,1,2};
   WORD	  work_out[57];
   EVNTREC	er;
 
 	NOT_USED(arg);
 
-	Rdwrs_operation(MSTARTREAD);
+	menu.area.height = globals.clheight + 3;
 
-  menu.tree = Appl_get_menu(TOP_MENU_OWNER);
+  evntglbl.mousefd = (WORD)Fopen(globals.mousename,0);
+  
+  if(evntglbl.mousefd < 0) {
+  	DB_printf("Couldn't open mouse pipe! Error %d",evntglbl.mousefd);
+  };
   
-	menu.area.height = menu.tree[menu.tree[0].ob_head].ob_height;
-
-	Rdwrs_operation(MENDREAD);
-
-  evntglbl.mousefd = (WORD)Fopen("u:\\dev\\aesmouse",0);
   evntglbl.lasttime = globals.time;
-  
+
   /* open up a vdi workstation to use in the process */
   
   evntglbl.evid = globals.vid;
@@ -1812,11 +1588,15 @@ static WORD mouseproc(LONG arg) {
       break;	
       
     case APPEVNT_MOUSE	:
-      globals.mouse_x = er.ap_value & 0xffff;
-      globals.mouse_y = er.ap_value >> 16;
+      globals.mouse_x = (WORD)(er.ap_value & 0xffff);
+      globals.mouse_y = (WORD)(er.ap_value >> 16);
 
-			if((mctrl.apid >= 0) && (mctrl.mode == MCTRL_COMPLETE)) {
-		    Appl_put_event((WORD)mctrl.apid,&er);
+			if(mctrl.apid >= 0) {
+	      check_rectevents(globals.mouse_x,globals.mouse_y);
+
+				if(mctrl.mode) {
+		    	Appl_put_event((WORD)mctrl.apid,&er);
+		    };
   		}
 			else {
 				if(globals.mouse_y < menu.area.height) {
@@ -1836,7 +1616,9 @@ static WORD mouseproc(LONG arg) {
     		
     		Rdwrs_operation(ASTARTREAD);
     		
-    		if((al = globals.ap_pri)) {
+    		al = globals.ap_pri;
+    		
+    		if(al) {
     			while(al->next) {
     				al = al->next;
     				
@@ -1878,23 +1660,18 @@ static WORD mouseproc(LONG arg) {
 void                   /*                                                   */
 Evhd_init_module(void) /*                                                   */
 /****************************************************************************/
-{
-  evntglbl.mouseid = (WORD)newfork(mouseproc,0,"EvntHndl");
-  
+{	
   /* Initialize internal mouse variables */
   
   Vdi_vq_mouse(globals.vid,&globals.mouse_button,&globals.mouse_x,&globals.mouse_y);
   
   /* Create and get the mouse event semaphore */
   
-  Psemaphore(0,MOUSESEM,-1);
-  
-  /* Release it */
-  
-  Psemaphore(3,MOUSESEM,-1);
+  mouse_sem = Rdwrs_create_sem();
 
-  Psemaphore(0,MCTRL_SEM,-1);
-  Psemaphore(3,MCTRL_SEM,-1);
+	mctrl.id = Rdwrs_create_sem();
+	
+  evntglbl.mouseid = (WORD)newfork(evnt_handler,0,"EvntHndl");
 }
 
 /****************************************************************************
@@ -1905,22 +1682,14 @@ void                   /*                                                   */
 Evhd_exit_module(void) /*                                                   */
 /****************************************************************************/
 {
-  /* Get the mouse event semaphore */
-  
-  Psemaphore(2,MOUSESEM,-1);
-  
-  /* Destroy it */
-  
-  Psemaphore(1,MOUSESEM,-1);
-  
-  Psemaphore(2,MCTRL_SEM,-1);
-  Psemaphore(1,MCTRL_SEM,-1);
-
 	Rdwrs_operation(MSTARTWRITE);
 
   (void)Pkill(evntglbl.mouseid,SIGKILL);
 
 	Rdwrs_operation(MENDWRITE);
+
+	Rdwrs_destroy_sem(mouse_sem);
+	Rdwrs_destroy_sem(mctrl.id);
 }
 
 /****************************************************************************
@@ -1934,13 +1703,13 @@ RECTEVENT *re)         /* Description of events that should be reported.    */
 {
   REVENTLIST *rl = (REVENTLIST *)Mxalloc(sizeof(REVENTLIST),GLOBALMEM);
   
-  Psemaphore(2,MOUSESEM,-1);
+  Psemaphore(2,mouse_sem,-1);
   
   rl->event = *re;
   rl->next = evntglbl.eventlist;
   evntglbl.eventlist = rl;
   
-  Psemaphore(3,MOUSESEM,-1);
+  Psemaphore(3,mouse_sem,-1);
   
   check_rectevents(globals.mouse_x,globals.mouse_y);
 }
@@ -1956,7 +1725,7 @@ WORD apid)             /* Application id to end reporting to.               */
 {
   REVENTLIST	**rl;
   
-  Psemaphore(2,MOUSESEM,-1);
+  Psemaphore(2,mouse_sem,-1);
   
   rl = &evntglbl.eventlist;
   
@@ -1973,7 +1742,7 @@ WORD apid)             /* Application id to end reporting to.               */
     rl = &(*rl)->next;
   };
   
-  Psemaphore(3,MOUSESEM,-1);
+  Psemaphore(3,mouse_sem,-1);
 }
 
 /****************************************************************************
@@ -1983,17 +1752,21 @@ WORD apid)             /* Application id to end reporting to.               */
 WORD              /*                                                        */
 Evhd_gain_mctrl(  /*                                                        */
 WORD apid,        /* Application to gain exclusive right.                   */
-WORD mode)        /* MCTRL_COMPLETE or MCTRL_BUTTON.                        */
+WORD send_coord)  /* 1 => movement packets will be sent, 0 => no movement.  */
 /****************************************************************************/
 {
 	WORD r;
 
-	if((mctrl.count > 0) && (mctrl.apid != apid))
+	if((mctrl.count > 0) && (mctrl.pid != Pgetpid())) {
 		return 0;
+	};
 	
-	r = Rdwrs_get_sem(apid, &mctrl);
-	if(r)
-		mctrl.mode = mode;
+	r = Rdwrs_get_sem(&mctrl);
+
+	if(r) {
+		mctrl.apid = apid;
+		mctrl.mode = send_coord;
+	};
 
 	return r;
 }
@@ -2002,12 +1775,13 @@ WORD mode)        /* MCTRL_COMPLETE or MCTRL_BUTTON.                        */
  * Evhd_release_mctrl                                                       *
  *  Release right to mouse events.                                          *
  ****************************************************************************/
-WORD                 /*                                                     */
-Evhd_release_mctrl(  /*                                                     */
-WORD apid)           /* Application to release exclusive right.             */
+WORD                      /*                                                */
+Evhd_release_mctrl(void)  /*                                                */
 /****************************************************************************/
 {
-	return Rdwrs_rel_sem( apid, &mctrl);
+	mctrl.apid = -1;
+	
+	return Rdwrs_rel_sem(&mctrl);
 }
 
 /****************************************************************************
