@@ -28,21 +28,21 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
-#define	__TCC_COMPAT__
-
-#include <gemfast.h>
 #include <mintbind.h>
 #include <stdio.h>
 
 #include "appl.h"
 #include "debug.h"
 #include "evnt.h"
+#include "evnthndl.h"
 #include "form.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "graf.h"
-#include "memdefs.h"
+#include "mintdefs.h"
 #include "objc.h"
 #include "resource.h"
+#include "rsrc.h"
 #include "types.h"
 #include "wind.h"
 
@@ -63,13 +63,14 @@
 WORD           /* Object that was selected.                                 */
 Form_do_do(    /*                                                           */
 WORD apid,     /* Application id.                                           */
+WORD vid,      /* VDI workstation id.                                       */
+WORD eventpipe,/* Event message pipe.                                       */
 OBJECT *tree,  /* Resource tree.                                            */
 WORD editobj)  /* Position of edit cursor.                                  */
 /****************************************************************************/
 {
 	WORD buffer[16];
 	WORD object,newobj,keyout;
-	WORD vid = Appl_get_vid(apid);
 	WORD idx;
 	
 	EVENTIN	ei = 	{
@@ -91,18 +92,21 @@ WORD editobj)  /* Position of edit cursor.                                  */
 		Objc_do_edit(vid,tree,editobj,0,&idx,ED_INIT);
 	};
 
+	Evhd_gain_mctrl(apid,0);
+	
 	while(1) {
-		Evnt_do_multi(apid,&ei,(COMMSG *)buffer,&eo,0);
+		Evnt_do_multi(apid,eventpipe,-1,&ei,(COMMSG *)buffer,&eo,0);
 
 		if(eo.events & MU_BUTTON) {
 			object = Objc_do_find(tree,0,9,eo.mx,eo.my,0);
 
 			if(object >= 0) {
-				if(!Form_do_button(apid,tree,object,eo.mc,&newobj)) {
+				if(!Form_do_button(apid,vid,eventpipe,tree,object,eo.mc,&newobj)) {
 					if(editobj != 0) {
 						Objc_do_edit(vid,tree,editobj,0,&idx,ED_END);
 					};
 
+					Evhd_release_mctrl();
 					return newobj;
 				}
 				else {
@@ -125,6 +129,7 @@ WORD editobj)  /* Position of edit cursor.                                  */
 					Objc_do_edit(vid,tree,editobj,0,&idx,ED_END);
 				};
 
+				Evhd_release_mctrl();
 				return newobj;
 			}
 			else if(newobj != editobj) {
@@ -148,32 +153,23 @@ WORD editobj)  /* Position of edit cursor.                                  */
 }
 
 void	Form_do(AES_PB *apb) {
-	apb->int_out[0] = Form_do_do(apb->global[2],(OBJECT *)apb->addr_in[0]
-		,apb->int_in[0]);		
+	apb->int_out[0] = Form_do_do(apb->global->apid,
+		apb->global->int_info->vid,
+		apb->global->int_info->eventpipe,
+		(OBJECT *)apb->addr_in[0],
+		apb->int_in[0]);		
 }
 
 /*form_dial 0x0033*/
 
-WORD	Form_do_dial(WORD apid,WORD mode,RECT *r1,RECT *r2) {
-	WORD vid = Appl_get_vid(apid);
-	
+WORD	Form_do_dial(WORD apid,WORD vid,WORD mode,RECT *r1,RECT *r2) {
 	switch(mode) {
 		case	FMD_GROW		:	/*0x0001*/
-			Wind_beg_update(apid);
+			Wind_beg_update();
 			Graf_do_grmobox(vid,r1,r2);
-			Wind_end_update(apid);
+			Wind_end_update();
 			
-			Rdwrs_operation(WSTARTWRITE);
-			
-			if((globals.win_vis->win->status & WIN_DIALOG)
-				&& (globals.win_vis->win->owner == apid)) {
-
-				Rdwrs_operation(WENDWRITE);
-				
-				return 1;				
-			};
-			
-			Rdwrs_operation(WENDWRITE);
+			return 1;
 					
 		case	FMD_START	:	/*0x0000*/
 			if((r2->width > 0) && (r2->height > 0)) {
@@ -204,10 +200,9 @@ WORD	Form_do_dial(WORD apid,WORD mode,RECT *r1,RECT *r2) {
 			};
 
 		case	FMD_SHRINK	:	/*0x0002*/
-			vid = Appl_get_vid(apid);
-			Wind_beg_update(apid);
+			Wind_beg_update();
 			Graf_do_grmobox(vid,r2,r1);
-			Wind_end_update(apid);
+			Wind_end_update();
 
 		case	FMD_FINISH	:	/*0x0003*/
 			Rdwrs_operation(WSTARTWRITE);
@@ -233,13 +228,14 @@ WORD	Form_do_dial(WORD apid,WORD mode,RECT *r1,RECT *r2) {
 }
 
 void	Form_dial(AES_PB *apb) {
-	apb->int_out[0] = Form_do_dial(apb->global[2],apb->int_in[0],
+	apb->int_out[0] = Form_do_dial(apb->global->apid,
+		apb->global->int_info->vid,apb->int_in[0],
 		(RECT *)&apb->int_in[1],(RECT *)&apb->int_in[5]);
 };
 
 /*form_alert 0x0034*/
 
-WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
+WORD	do_form_alert(WORD apid,WORD vid,WORD eventpipe,WORD def,BYTE *alertstring) {
 	BYTE	*s = (BYTE *)Mxalloc(strlen(alertstring) + 1,PRIVATEMEM);
 	
 	WORD	i = 0;
@@ -253,15 +249,13 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 	OBJECT	*tree;
 	TEDINFO	*ti;
 	
-	RECT	clip;
+	RECT	 clip;
 	
-	BYTE	*icon,*text,*buttons;
+	BYTE	 *icon,*text,*buttons;
 
-	WORD	textwidth = 0,buttonwidth = 0;
+	WORD	 textwidth = 0,buttonwidth = 0;
 	
-	WORD vid = Appl_get_vid(apid);
-	
-	Graf_do_handle(apid,&cwidth,&cheight,&width,&height);
+	Graf_do_handle(&cwidth,&cheight,&width,&height);
 	
 	strcpy(s,alertstring);
 	
@@ -311,21 +305,21 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 		
 	s[i] = 0;
 	
-	tree = (OBJECT *)Mxalloc((1 + no_butts + no_rows) * sizeof(OBJECT)
+	tree = (OBJECT *)Mxalloc((2 + no_butts + no_rows) * sizeof(OBJECT)
 				,PRIVATEMEM);
 	
 	ti = (TEDINFO *)Mxalloc(no_rows * sizeof(TEDINFO),PRIVATEMEM);
 	
-	memcpy(&tree[0],&alerttad[0],sizeof(OBJECT));
+	memcpy(&tree[0],&globals.alerttad[0],sizeof(OBJECT));
 	
 	tree[0].ob_head = -1;
 	tree[0].ob_tail = -1;
 	
 	for(i = 0; i < no_rows; i ++) {
-		memcpy(&tree[1 + i],&alerttad[AL_TEXT],sizeof(OBJECT));
-		memcpy(&ti[i],alerttad[AL_TEXT].ob_spec.tedinfo
+		memcpy(&tree[1 + i],&globals.alerttad[AL_TEXT],sizeof(OBJECT));
+		memcpy(&ti[i],globals.alerttad[AL_TEXT].ob_spec.tedinfo
 			,sizeof(TEDINFO));
-		tree[i + 1].ob_width = strlen(text) * cwidth;
+		tree[i + 1].ob_width = (WORD)(strlen(text) * cwidth);
 		tree[i + 1].ob_height = globals.clheight;
 		tree[i + 1].ob_spec.tedinfo = &ti[i];
 	
@@ -345,7 +339,7 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 	};
 
 	for(i = 0; i < no_butts; i ++) {
-		memcpy(&tree[1 + i + no_rows],&alerttad[AL_BUTTON],sizeof(OBJECT));
+		memcpy(&tree[1 + i + no_rows],&globals.alerttad[AL_BUTTON],sizeof(OBJECT));
 	
 		tree[i + 1 + no_rows].ob_y = no_rows * globals.clheight + 20;
 
@@ -368,23 +362,57 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 		do_objc_add(tree,0,i + 1 + no_rows);
 	};
 	
+	memcpy(&tree[1 + no_butts + no_rows],&globals.alerttad[AL_ICON],sizeof(OBJECT));
+
+	do_objc_add(tree,0,1 + no_butts + no_rows);
+
+	switch(*icon) {
+	case '1':
+		tree[1 + no_butts + no_rows].ob_spec.index = globals.aiconstad[AIC_EXCLAMATION].ob_spec.index;
+		break;
+
+	case '2':
+		tree[1 + no_butts + no_rows].ob_spec.index = globals.aiconstad[AIC_QUESTION].ob_spec.index;
+		break;
+
+	case '3':
+		tree[1 + no_butts + no_rows].ob_spec.index = globals.aiconstad[AIC_STOP].ob_spec.index;
+		break;
+
+	case '4':
+		tree[1 + no_butts + no_rows].ob_spec.index = globals.aiconstad[AIC_INFO].ob_spec.index;
+		break;
+
+	case '5':
+		tree[1 + no_butts + no_rows].ob_spec.index = globals.aiconstad[AIC_DISK].ob_spec.index;
+		break;
+
+	default:
+	tree[1 + no_butts + no_rows].ob_flags |= HIDETREE;		
+	};
+		
 	buttonwidth += 2;
 
 	if(def) {
 		tree[no_rows + def].ob_flags |= DEFAULT;
 	};
 	
-	tree[no_rows + no_butts].ob_flags |= LASTOB;
+	tree[no_rows + no_butts + 1].ob_flags |= LASTOB;
 
 	tree[0].ob_width = (buttonwidth + 10) * no_butts + 10;
 	
-	if(textwidth + 20 > tree[0].ob_width)
-		tree[0].ob_width = textwidth + 20;
+	if(textwidth + 28 + tree[1 + no_butts + no_rows].ob_width
+			> tree[0].ob_width) {
+		tree[0].ob_width = textwidth + 28 +
+			tree[1 + no_butts + no_rows].ob_width;
+	};
 	
 	tree[0].ob_height = globals.clheight * no_rows + 45;
 	
 	for(i = 0; i < no_rows; i++) {
-		tree[i + 1].ob_x = (tree[0].ob_width - textwidth) >> 1;
+		tree[i + 1].ob_x = (tree[0].ob_width - textwidth - 
+			tree[1 + no_butts + no_rows].ob_width) / 2 +
+			tree[1 + no_butts + no_rows].ob_width + 8;
 		tree[i + 1].ob_y = i * globals.clheight + 10;
 	};
 	
@@ -397,13 +425,13 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 	
 	Form_do_center(tree,&clip);
 
-	Form_do_dial(apid,FMD_START,&clip,&clip);
+	Form_do_dial(apid,vid,FMD_START,&clip,&clip);
 
 	Objc_do_draw(vid,tree,0,9,&clip);
+
+	but_chosen = Form_do_do(apid,vid,eventpipe,tree,0) & 0x7fff;
 	
-	but_chosen = Form_do_do(apid,tree,0) & 0x7fff;
-	
-	Form_do_dial(apid,FMD_FINISH,&clip,&clip);
+	Form_do_dial(apid,vid,FMD_FINISH,&clip,&clip);
 
 	Mfree(ti);
 	Mfree(tree);
@@ -413,46 +441,67 @@ WORD	do_form_alert(WORD apid,WORD def,BYTE *alertstring) {
 }
 
 void	Form_alert(AES_PB *apb) {
-	apb->int_out[0] = do_form_alert(apb->global[2],apb->int_in[0]
-								,(BYTE *)apb->addr_in[0]);
+	apb->int_out[0] = do_form_alert(apb->global->apid,
+											apb->global->int_info->vid,
+											apb->global->int_info->eventpipe,
+											apb->int_in[0],
+											(BYTE *)apb->addr_in[0]);
 }
 
 /*form_error 0x0035*/
 
-void	Form_error(AES_PB *apb) {
+/****************************************************************************
+ *  Form_do_error                                                           *
+ *   Display pre-defined error alert box.                                   *
+ ****************************************************************************/
+WORD              /* Exit button.                                           */
+Form_do_error(    /*                                                        */
+WORD   apid,      /* Application id number.                                 */
+WORD   vid,       /* VDI workstation id.                                    */
+WORD   eventpipe, /* Event message pipe.                                    */
+WORD   error)     /* Error code.                                            */
+/****************************************************************************/
+{
 	BYTE	s[100];
 	BYTE	*sp = s;
 	
-	switch(apb->int_in[0])
+	switch(error)
 	{
 		case	FERR_FILENOTFOUND:
 		case	FERR_PATHNOTFOUND:
 		case	FERR_NOFILES:
-			sp = fr_string[ERROR_2_3_18];
+			sp = globals.fr_string[ERROR_2_3_18];
 			break;
 		case	FERR_NOHANDLES:
-			sp = fr_string[ERROR_4];
+			sp = globals.fr_string[ERROR_4];
 			break;
 		case	FERR_ACCESSDENIED:
-			sp = fr_string[ERROR_5];
+			sp = globals.fr_string[ERROR_5];
 			break;
 		case	FERR_LOWMEM:
 		case	FERR_BADENVIRON:
 		case	FERR_BADFORMAT:
-			sp = fr_string[ERROR_8_10_11];
+			sp = globals.fr_string[ERROR_8_10_11];
 			break;
 		case	FERR_BADDRIVE:
-			sp = fr_string[ERROR_15];
+			sp = globals.fr_string[ERROR_15];
 			break;
 		case	FERR_DELETEDIR:
-			sp = fr_string[ERROR_16];
+			sp = globals.fr_string[ERROR_16];
 			break;
 		default:
-			sprintf(s,fr_string[ERROR_GENERAL],apb->int_in[0]);
+			sprintf(s,globals.fr_string[ERROR_GENERAL],error);
 	};
 	
-	apb->int_out[0] = do_form_alert(apb->global[2],1,sp);
-};
+	return do_form_alert(apid,vid,eventpipe,1,sp);
+}
+
+void	Form_error(AES_PB *apb) {
+	apb->int_out[0] = Form_do_error(apb->global->apid,
+											apb->global->int_info->vid,
+											apb->global->int_info->eventpipe,
+											apb->int_in[0]);
+}
 
 /*form_center 0x0036*/
 
@@ -571,9 +620,8 @@ Form_keybd(       /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	WORD vid = Appl_get_vid(apb->global[2]);
-
-	apb->int_out[0] = Form_do_keybd(vid,(OBJECT *)apb->addr_in[0],apb->int_in[0],
+	apb->int_out[0] = Form_do_keybd(apb->global->int_info->vid,
+										(OBJECT *)apb->addr_in[0],apb->int_in[0],
 										apb->int_in[1],&apb->int_out[1],
 										&apb->int_out[2]);
 }
@@ -588,6 +636,8 @@ AES_PB *apb)      /* AES parameter block.                                   */
 WORD            /* 0 if exit object was found or 1.                         */
 Form_do_button( /*                                                          */
 WORD apid,      /* Application id.                                          */
+WORD vid,       /* VDI workstation id.                                      */
+WORD eventpipe, /* Event message pipe.                                      */
 OBJECT *tree,   /* Resource tree.                                           */
 WORD obj,       /* Object to try the clicks on.                             */
 WORD clicks,    /* Number of clicks.                                        */
@@ -595,7 +645,6 @@ WORD *newobj)   /* Next object to gain edit focus, or 0.                    */
 /****************************************************************************/
 {
 	WORD	dummy;
-	WORD	vid = Appl_get_vid(apid);
 
 	*newobj = 0;
 
@@ -630,7 +679,7 @@ WORD *newobj)   /* Next object to gain edit focus, or 0.                    */
 					tree[i].ob_state |= SELECTED,REDRAW);
 			};
 			
-			Evnt_do_button(apid,0,LEFT_BUTTON,0,&dummy,&dummy,&dummy,&dummy);
+			Evnt_do_button(apid,eventpipe,0,LEFT_BUTTON,0,&dummy,&dummy,&dummy,&dummy);
 			
 			if(tree[obj].ob_flags & (TOUCHEXIT | EXIT)) {
 				*newobj = obj;
@@ -653,7 +702,7 @@ WORD *newobj)   /* Next object to gain edit focus, or 0.                    */
 				instate ^= SELECTED;
 			};
 	
-			if((Graf_do_watchbox(apid,tree,obj,instate,outstate) == 1) &&
+			if((Graf_do_watchbox(apid,vid,eventpipe,tree,obj,instate,outstate) == 1) &&
 					(tree[obj].ob_flags & (EXIT | TOUCHEXIT))) {
 	
 				*newobj = obj;
@@ -688,6 +737,9 @@ WORD *newobj)   /* Next object to gain edit focus, or 0.                    */
 }
 
 void	Form_button(AES_PB *apb) {
-	apb->int_out[0] = Form_do_button(apb->global[2],(OBJECT *)apb->addr_in[0]
-		,apb->int_in[0],apb->int_in[1],&apb->int_out[1]);
+	apb->int_out[0] = Form_do_button(apb->global->apid,
+												apb->global->int_info->vid,
+												apb->global->int_info->eventpipe,
+												(OBJECT *)apb->addr_in[0],
+												apb->int_in[0],apb->int_in[1],&apb->int_out[1]);
 }
