@@ -1,8 +1,6 @@
 #ifndef	__GLOBAL__
 #define	__GLOBAL__
 
-#include	<aesbind.h>
-
 #include	"types.h"
 
 #define	STKSIZE	4096
@@ -16,6 +14,7 @@ typedef struct globalvars {
 	WORD    realmove;   /* if set realtime moving of windows is enabled */
 	WORD    realsize;
   WORD    realslide;  /* if set realtime sliding of windows is enabled */
+	WORD    wind_appl;  /* show application name in window mover */
 
 	WORD    num_pens;   /* number of available vdi pens */
 
@@ -49,6 +48,19 @@ typedef struct globalvars {
 
 	WORD    icon_width;   /* Width of iconified window */
 	WORD    icon_height;  /* Height of iconified window */
+
+	OBJECT  *aiconstad;
+	OBJECT  *alerttad;
+	OBJECT  *deskbgtad;
+	OBJECT  *fiseltad;
+	OBJECT  *informtad;
+	OBJECT  *menutad;
+	OBJECT  *mouseformstad;
+	OBJECT  *pmenutad;
+	OBJECT  *windowtad;
+	BYTE    **fr_string;
+
+	BYTE    mousename[30];
 }GLOBALVARS;
 
 extern GLOBALVARS	globals;
