@@ -30,7 +30,6 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
-#include <aesbind.h>
 #include <mintbind.h>
 #include <osbind.h>
 #include <stdio.h>
@@ -39,11 +38,13 @@
 #include "debug.h"
 #include "evnt.h"
 #include "evnthndl.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "graf.h"
 #include "misc.h"
 #include "objc.h"
 #include "resource.h"
+#include "rsrc.h"
 #include "types.h"
 #include "vdi.h"
 #include "wind.h"
@@ -52,13 +53,14 @@
  * Macros                                                                   *
  ****************************************************************************/
 
-#define GRAF_STEPS 10
+#define GRAF_STEPS 15
 
 /****************************************************************************
  * Module global variables                                                  *
  ****************************************************************************/
 
-static MFORM m_arrow,m_text_crsr,m_busy_bee,m_point_hand,m_flat_hand;
+static MFORM m_arrow,m_text_crsr,m_busy_bee,m_point_hand,m_flat_hand,
+             m_thin_cross,m_thick_cross,m_outln_cross;
 static MFORM current,last,last_saved;
 
 /****************************************************************************
@@ -77,8 +79,8 @@ void icon2mform(MFORM *mf,OBJECT *icon) {
 	mf->mf_bg = 0;
 	
 	for(i = 0; i < 16; i++) {
-		mf->mf_mask[i] = ((ICONBLK *)icon->ob_spec)->ib_pmask[i];
-		mf->mf_data[i] = ((ICONBLK *)icon->ob_spec)->ib_pdata[i];
+		mf->mf_mask[i] = icon->ob_spec.iconblk->ib_pmask[i];
+		mf->mf_data[i] = icon->ob_spec.iconblk->ib_pdata[i];
 	};
 }
 
@@ -86,18 +88,151 @@ void icon2mform(MFORM *mf,OBJECT *icon) {
  * Public functions                                                         *
  ****************************************************************************/
  
-void	Graf_init_module(void) {
-	icon2mform(&m_arrow,&mouseformstad[MARROW]);
-	icon2mform(&m_text_crsr,&mouseformstad[MTEXT_CRSR]);
-	icon2mform(&m_busy_bee,&mouseformstad[MBUSY_BEE]);
-	icon2mform(&m_point_hand,&mouseformstad[MPOINT_HAND]);
-	icon2mform(&m_flat_hand,&mouseformstad[MFLAT_HAND]);
+void	Graf_init_module(void) {	
+	icon2mform(&m_arrow,&globals.mouseformstad[MARROW]);
+	icon2mform(&m_text_crsr,&globals.mouseformstad[MTEXT_CRSR]);
+	icon2mform(&m_busy_bee,&globals.mouseformstad[MBUSY_BEE]);
+	icon2mform(&m_point_hand,&globals.mouseformstad[MPOINT_HAND]);
+	icon2mform(&m_flat_hand,&globals.mouseformstad[MFLAT_HAND]);
+	icon2mform(&m_thin_cross,&globals.mouseformstad[MTHIN_CROSS]);
+	icon2mform(&m_thick_cross,&globals.mouseformstad[MTHICK_CROSS]);
+	icon2mform(&m_outln_cross,&globals.mouseformstad[MOUTLN_CROSS]);
 	
 	current = m_arrow;
 	last = m_arrow;
 	last_saved = m_arrow;
 }
 
+/* 0x0046 graf_rubberbox() */
+
+/****************************************************************************
+ * Graf_do_rubberbox                                                        *
+ *  Implementation of graf_rubberbox().                                     *
+ ****************************************************************************/
+WORD                /* 1 if ok or 0.                                        */
+Graf_do_rubberbox(  /*                                                      */
+WORD   vid,         /* VDI workstation id.                                  */
+WORD   eventpipe,   /* Event message pipe handle.                           */
+WORD   bx,          /* Top left corner x.                                   */
+WORD   by,          /* Top left corner y.                                   */
+WORD   minw,        /* Minimum width.                                       */
+WORD   minh,        /* Minimum height.                                      */
+WORD   *endw,       /* Final width.                                         */
+WORD   *endh)       /* Final height.                                        */
+/****************************************************************************/
+{
+	EVNTREC er;
+	WORD    xyarray[10];
+	WORD    lastw = globals.mouse_x - bx,lasth = globals.mouse_y - by;
+	WORD    neww,newh;
+
+	if(!(globals.mouse_button & LEFT_BUTTON)) {	
+		*endw = globals.mouse_x - bx;
+		*endh = globals.mouse_y - by;
+
+		if(*endw < minw) {
+			*endw = minw;
+		};
+
+		if(*endh < minh) {
+			*endh = minh;
+		};
+	
+		return 1;
+	};
+	
+	Vdi_vswr_mode(vid,MD_XOR);
+  Vdi_vsl_type(vid,DOTTED);
+
+  xyarray[0] = bx;
+  xyarray[1] = by;
+  xyarray[2] = bx + lastw - 1;
+  xyarray[3] = xyarray[1];
+  xyarray[4] = xyarray[2];
+  xyarray[5] = by + lasth - 1;
+  xyarray[6] = xyarray[0];
+  xyarray[7] = xyarray[5];
+  xyarray[8] = xyarray[0];
+  xyarray[9] = xyarray[1];
+	 
+	Vdi_v_hide_c(vid);
+  Vdi_v_pline(vid,5,xyarray);
+  Vdi_v_show_c(vid,1);
+	
+	while(Fread(eventpipe,sizeof(EVNTREC),&er)) {
+		if(er.ap_event == APPEVNT_MOUSE) {
+			neww = (WORD)((er.ap_value & 0xffff) - bx);
+  		newh = (WORD)((er.ap_value >> 16) - by);
+
+			if(neww < minw) {
+				neww = minw;
+			};
+
+			if(newh < minh) {
+				newh = minh;
+			};
+
+			if((lastw != neww) || (lasth != newh)) {
+	      Vdi_v_hide_c(vid);
+	      Vdi_v_pline(vid,5,xyarray);
+
+	      lastw = neww;
+	      lasth = newh;
+
+			  xyarray[0] = bx;
+			  xyarray[1] = by;
+			  xyarray[2] = bx + lastw - 1;
+			  xyarray[3] = xyarray[1];
+			  xyarray[4] = xyarray[2];
+			  xyarray[5] = by + lasth - 1;
+			  xyarray[6] = xyarray[0];
+			  xyarray[7] = xyarray[5];
+			  xyarray[8] = xyarray[0];
+			  xyarray[9] = xyarray[1];
+	 
+	      Vdi_v_pline(vid,5,xyarray);
+	      Vdi_v_show_c(vid,1);
+	    };
+		};
+		
+		if(er.ap_event == APPEVNT_BUTTON) {
+			if(!(er.ap_value & LEFT_BUTTON)) {
+				break;
+			};
+		};
+	};
+
+  Vdi_v_hide_c(vid);
+  Vdi_v_pline(vid,5,xyarray);
+	Vdi_v_show_c(vid,1);
+
+	*endw = lastw;
+	*endh = lasth;
+	
+	return 1;
+}
+
+/****************************************************************************
+ * Graf_rubberbox                                                           *
+ *  0x0046 graf_rubberbox().                                                *
+ ****************************************************************************/
+void              /*                                                        */
+Graf_rubberbox(   /*                                                        */
+AES_PB *apb)      /* AES parameter block.                                   */
+/****************************************************************************/
+{
+	Evhd_gain_mctrl(apb->global->apid,TRUE);
+
+	apb->int_out[0] = Graf_do_rubberbox(apb->global->int_info->vid,
+										apb->global->int_info->eventpipe,apb->int_in[0],
+										apb->int_in[1],apb->int_in[2],apb->int_in[3],
+										&apb->int_out[1],&apb->int_out[2]);
+
+	Evhd_release_mctrl();
+}
+
+
+/* 0x0071 graf_dragbox() */
 
 /****************************************************************************
  * Graf_do_dragbox                                                          *
@@ -105,7 +240,8 @@ void	Graf_init_module(void) {
  ****************************************************************************/
 WORD              /* 1 if ok or 0.                                          */
 Graf_do_dragbox(  /*                                                        */
-WORD   apid,      /* Application id.                                        */
+WORD   vid,       /* VDI workstation id.                                    */
+WORD   eventpipe, /* Event message pipe handle.                             */
 WORD   w,         /* Width of box.                                          */
 WORD   h,         /* Height of box.                                         */
 WORD   sx,        /* Starting x.                                            */
@@ -115,8 +251,6 @@ WORD   *endx,     /* Ending x.                                              */
 WORD   *endy)     /* Ending y.                                              */
 /****************************************************************************/
 {
-	WORD    eventpipe = Appl_get_eventpipe(apid);
-	WORD    vid = Appl_get_vid(apid);
 	EVNTREC er;
 	WORD    relx = sx - globals.mouse_x;
 	WORD    rely = sy - globals.mouse_y;
@@ -130,8 +264,6 @@ WORD   *endy)     /* Ending y.                                              */
 	
 		return 1;
 	};
-	
-	Evhd_gain_mctrl(apid,MCTRL_COMPLETE);
 
 	Vdi_vswr_mode(vid,MD_XOR);
   Vdi_vsl_type(vid,DOTTED);
@@ -153,8 +285,8 @@ WORD   *endy)     /* Ending y.                                              */
 	
 	while(Fread(eventpipe,sizeof(EVNTREC),&er)) {
 		if(er.ap_event == APPEVNT_MOUSE) {
-			newx = (er.ap_value & 0xffff) + relx;
-  		newy = (er.ap_value >> 16) + rely;
+			newx = (WORD)((er.ap_value & 0xffff) + relx);
+  		newy = (WORD)((er.ap_value >> 16) + rely);
 
 			if(newx < bound->x) {
 				newx = bound->x;
@@ -203,8 +335,6 @@ WORD   *endy)     /* Ending y.                                              */
   Vdi_v_hide_c(vid);
   Vdi_v_pline(vid,5,xyarray);
 	Vdi_v_show_c(vid,1);
-	
-	Evhd_release_mctrl(apid);
 
 	*endx = lastx;
 	*endy = lasty;
@@ -221,10 +351,15 @@ Graf_dragbox(     /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	apb->int_out[0] = Graf_do_dragbox(apb->global[2],apb->int_in[0],
+	Evhd_gain_mctrl(apb->global->apid,TRUE);
+
+	apb->int_out[0] = Graf_do_dragbox(apb->global->int_info->vid,
+										apb->global->int_info->eventpipe,apb->int_in[0],
 										apb->int_in[1],apb->int_in[2],apb->int_in[3],
 										(RECT *)&apb->int_in[4],&apb->int_out[1],
 										&apb->int_out[2]);
+
+	Evhd_release_mctrl();
 }
 
 
@@ -278,10 +413,6 @@ RECT *r2)         /* End rectangle.                                         */
 			xy[7] = xy[5];
 			xy[8] = xy[0];
 			xy[9] = xy[1];
-			
-			if((f % 3) == 2) {
-				Vsync();
-			};
 		}
 	}
 	
@@ -301,34 +432,23 @@ Graf_movebox(     /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	WORD vid = Appl_get_vid(apb->global[2]);
+	RECT r1,r2;
 	
-	if(vid >= 0) {
-		RECT r1,r2;
-		
-		r1.x = apb->int_in[2];
-		r1.y = apb->int_in[3];
-		r1.width = apb->int_in[0];
-		r1.height = apb->int_in[1];
-		
-		r2.x = apb->int_in[4];
-		r2.y = apb->int_in[5];
-		r2.width = apb->int_in[0];
-		r2.height = apb->int_in[1];
-		
-		Wind_beg_update(apb->global[2]);
-		
-		apb->int_out[0] = Graf_do_grmobox(vid,&r1,&r2);
-		
-		Wind_end_update(apb->global[2]);
-	}
-	else {
-		DB_printf("%s: Line %d: Graf_movebox\r\n"
-							"Couldn't find application description.",
-							__FILE__,__LINE__);
+	r1.x = apb->int_in[2];
+	r1.y = apb->int_in[3];
+	r1.width = apb->int_in[0];
+	r1.height = apb->int_in[1];
+	
+	r2.x = apb->int_in[4];
+	r2.y = apb->int_in[5];
+	r2.width = apb->int_in[0];
+	r2.height = apb->int_in[1];
 
-		apb->int_out[0] = 0;
-	};
+	Wind_beg_update();
+	
+	apb->int_out[0] = Graf_do_grmobox(apb->global->int_info->vid,&r1,&r2);
+	
+	Wind_end_update();
 }
 
 /****************************************************************************
@@ -340,23 +460,13 @@ Graf_growbox(     /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	WORD vid = Appl_get_vid(apb->global[2]);
-	
-	if(vid >= 0) {
-		Wind_beg_update(apb->global[2]);
+	Wind_beg_update();
 
-		apb->int_out[0] = Graf_do_grmobox(vid,(RECT *)&apb->int_in[0],
-																					(RECT *)&apb->int_in[4]);
+	apb->int_out[0] = Graf_do_grmobox(apb->global->int_info->vid,
+																		(RECT *)&apb->int_in[0],
+																		(RECT *)&apb->int_in[4]);
 
-		Wind_end_update(apb->global[2]);
-	}
-	else {
-		DB_printf("%s: Line %d: Graf_growbox\r\n"
-							"Couldn't find application description.",
-							__FILE__,__LINE__);
-
-		apb->int_out[0] = 0;
-	};
+	Wind_end_update();
 }
 
 /****************************************************************************
@@ -378,6 +488,8 @@ AES_PB *apb)      /* AES parameter block.                                   */
 WORD              /* 1 if inside object when button was released or 0.      */
 Graf_do_watchbox( /*                                                        */
 WORD   apid,      /* Application id.                                        */
+WORD   vid,       /* VDI workstation id.                                    */
+WORD   eventpipe, /* Event message pipe.                                    */
 OBJECT *tree,     /* Resource tree.                                         */
 WORD   obj,       /* Object to watch.                                       */
 WORD   instate,   /* State when inside object.                              */
@@ -401,8 +513,6 @@ WORD   outstate)  /* State when outside object.                             */
 	COMMSG    buffer;
 	RECT      clip;
 	
-	WORD vid = Appl_get_vid(apid);
-
 	Objc_calc_clip(tree,obj,&clip);
 	Objc_do_offset(tree,obj,(WORD *)&ei.m1r);
 	ei.m1r.width = tree[obj].ob_width;
@@ -419,10 +529,10 @@ WORD   outstate)  /* State when outside object.                             */
 		Objc_do_change(vid,tree,obj,&clip,outstate,REDRAW);
 	};
 	
-	Evhd_gain_mctrl(apid,MCTRL_BUTTON);
+	Evhd_gain_mctrl(apid,FALSE);
 	
 	while(1) {
-		Evnt_do_multi(apid,&ei,&buffer,&eo,0);
+		Evnt_do_multi(apid,eventpipe,-1,&ei,&buffer,&eo,0);
 		
 		if(eo.events & MU_BUTTON) {
 			break;
@@ -440,7 +550,7 @@ WORD   outstate)  /* State when outside object.                             */
 		};
 	};
 	
-	Evhd_release_mctrl(apid);
+	Evhd_release_mctrl();
 					
 	if(ei.m1flag == MO_LEAVE) {
 		return 1;
@@ -458,7 +568,10 @@ Graf_watchbox(    /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	apb->int_out[0] = Graf_do_watchbox(apb->global[2],(OBJECT *)apb->addr_in[0],
+	apb->int_out[0] = Graf_do_watchbox(apb->global->apid,
+										apb->global->int_info->vid,
+										apb->global->int_info->eventpipe,
+										(OBJECT *)apb->addr_in[0],
 										apb->int_in[1],apb->int_in[2],apb->int_in[3]);
 }
 
@@ -469,6 +582,8 @@ AES_PB *apb)      /* AES parameter block.                                   */
 WORD              /* Relative offset.                                       */
 Graf_do_slidebox( /*                                                        */
 WORD   apid,      /* Application id.                                        */
+WORD   vid,       /* VDI workstation id.                                    */
+WORD   eventpipe, /* Event message pipe.                                    */
 OBJECT *tree,     /* Resource tree.                                         */
 WORD   parent,    /* Parent object.                                         */
 WORD   obj,       /* Slider object.                                         */
@@ -486,8 +601,13 @@ WORD   orient)    /* Orientation. 0 => horizontal, 1 => vertical.           */
 	Objc_area_needed(tree,obj,&slid);
 	Objc_area_needed(tree,parent,&bound);
 	
-	Graf_do_dragbox(apid,slid.width,slid.height,
+	Evhd_gain_mctrl(apid,TRUE);
+	
+	Graf_do_dragbox(vid,eventpipe,
+									slid.width,slid.height,
 									slid.x,slid.y,&bound,&x,&y);
+									
+	Evhd_release_mctrl();
 	
 	if(orient == 0) {
 		if(tree[obj].ob_width != tree[parent].ob_width) {
@@ -514,24 +634,27 @@ Graf_slidebox(    /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	apb->int_out[0] = Graf_do_slidebox(apb->global[2],(OBJECT *)apb->addr_in[0],
+	apb->int_out[0] = Graf_do_slidebox(apb->global->apid,
+										apb->global->int_info->vid,
+										apb->global->int_info->eventpipe,
+										(OBJECT *)apb->addr_in[0],
 										apb->int_in[0],apb->int_in[1],apb->int_in[2]);
 }
 
 /*graf_handle 0x004d*/
 
-WORD	Graf_do_handle(WORD apid,WORD *cwidth,WORD *cheight,WORD *width
+void Graf_do_handle(WORD *cwidth,WORD *cheight,WORD *width
 							,WORD *height) {
 	*cwidth = globals.clwidth;
 	*cheight = globals.clheight;
 	*width = globals.blwidth;
 	*height = globals.blheight;
-
-	return Appl_get_vid(apid);
 }
 
 void	Graf_handle(AES_PB *apb) {
-	apb->int_out[0] = Graf_do_handle(apb->global[2],&apb->int_out[1]
+	apb->int_out[0] = apb->global->int_info->vid;
+	
+	Graf_do_handle(&apb->int_out[1]
 								,&apb->int_out[2],&apb->int_out[3]
 								,&apb->int_out[4]);	
 }
@@ -582,6 +705,30 @@ WORD Graf_do_mouse(WORD vid,WORD mode,MFORM *formptr) {
 				Vdi_v_show_c(vid,0);
 				break;
 			
+			case THIN_CROSS: /*0x005*/
+				last = current;
+				current = m_thin_cross;
+				Vdi_v_hide_c(vid);
+				Vdi_vsc_form(vid,&m_thin_cross);
+				Vdi_v_show_c(vid,0);
+				break;
+			
+			case THICK_CROSS: /*0x006*/
+				last = current;
+				current = m_thick_cross;
+				Vdi_v_hide_c(vid);
+				Vdi_vsc_form(vid,&m_thick_cross);
+				Vdi_v_show_c(vid,0);
+				break;
+			
+			case OUTLN_CROSS: /*0x007*/
+				last = current;
+				current = m_outln_cross;
+				Vdi_v_hide_c(vid);
+				Vdi_vsc_form(vid,&m_outln_cross);
+				Vdi_v_show_c(vid,0);
+				break;
+			
 			case USER_DEF	:
 				last = current;
 				current = *formptr;
@@ -637,7 +784,7 @@ WORD Graf_do_mouse(WORD vid,WORD mode,MFORM *formptr) {
 }
 
 void Graf_mouse(AES_PB *apb) {
-	apb->int_out[0] = Graf_do_mouse(Appl_get_vid(apb->global[2]),apb->int_in[0],(MFORM *)apb->addr_in[0]);
+	apb->int_out[0] = Graf_do_mouse(apb->global->int_info->vid,apb->int_in[0],(MFORM *)apb->addr_in[0]);
 }
 
 /*graf_mkstate 0x004f*/
