@@ -34,7 +34,6 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
-#include <aesbind.h>
 #include <basepage.h>
 #include <ctype.h>
 #include <mintbind.h>
@@ -48,17 +47,20 @@
 #include "boot.h"
 #include "debug.h"
 #include "evnt.h"
+#include "form.h"
+#include "fsel.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "lxgemdos.h"
-#include "memdefs.h"
+#include "mintdefs.h"
 #include "menu.h"
 #include "mesagdef.h"
 #include "misc.h"
 #include "objc.h"
 #include "rdwrs.h"
 #include "resource.h"
+#include "rsrc.h"
 #include "types.h"
-#include "version.h"
 #include "wind.h"
 
 #include <sysvars.h>
@@ -101,7 +103,7 @@ struct {
 
 static BYTE progpath[500],progfile[70];
 
-static BYTE versionstring[50];
+static GLOBAL_ARRAY global_array;
 
 /****************************************************************************
  * Local functions (use static!)                                            *
@@ -127,9 +129,10 @@ static WORD	create_menu_window(RECT *maxsize) {
 static WORD menu_init_windows(void) {
 	RECT	r;
 	
-	wind_get(0,WF_FULLXYWH, &r.x, &r.y, &r.width, &r.height);
+	Wind_do_get(0,WF_FULLXYWH, &r.x, &r.y, &r.width, &r.height);
 	
-	r.height = globals.clheight + 3;
+	r.height = r.y;
+	r.y = 0;
 
 	mglob.winbar = create_menu_window(&r);
 
@@ -141,21 +144,20 @@ static WORD menu_init_windows(void) {
 		return MENU_GEN_ERR;
 	};
 
-	wind_open(mglob.winbar, r.x, r.y, r.width, r.height);
+	Rdwrs_operation(WSTARTWRITE);
+
+	Wind_do_open(global_array.int_info->vid,mglob.winbar,&r);
+
+	Rdwrs_operation(WENDWRITE);
+
 
 	return 0;
 }
 
-static void menu_shutdown(void) {
-	wind_close(mglob.winbar);
-	wind_delete(mglob.winbar);
-}
-
-
-static void handle_redraw(REDRAWSTRUCT *m,WORD menubox) {
-	WORD   x,y,w,h;
+static void redraw_menu_bar(void) {
 	OBJECT *menu;
-	wind_update(BEG_UPDATE);
+
+	Wind_beg_update();
 
 	Rdwrs_operation(MSTARTREAD);
 	
@@ -166,57 +168,28 @@ static void handle_redraw(REDRAWSTRUCT *m,WORD menubox) {
 	Rdwrs_quick(AENDREAD);
 
 	if(menu) {
-		if(mglob.winbar == m->wid) {
-			WORD xn,yn,wn,hn,hp;
-			WORD yp = pmenutad[0].ob_y;
-			WORD deskbox = menu[menu[0].ob_tail].ob_head;
-			
-			wind_get(m->wid,WF_FIRSTXYWH,&x,&y,&w,&h);
+		RECT r;
 		
-			while((w > 0) && (h > 0)) {
-				if(deskbox == menubox) {
-					xn = max(x,m->area.x);
-					wn = min(x + w ,m->area.x + m->area.width) - xn;
-					yn = max(y,m->area.y);
-					hn = min(y + h, m->area.y + m->area.height) - yn;
-
-					yp = max(yn,yp);
-
-					hp = hn - yp + yn;
-					hn = min(hn,yp - yn);
-						
-					if((wn > 0) && (hn > 0)) {
-						objc_draw(menu,0,9,xn,yn,wn,hn);
-					};
-					
-					if((wn > 0) && (hp > 0)) {
-						objc_draw(pmenutad,0,9,xn,yp,wn,hp);
-					};
-				}
-				else {
-					xn = max(x,m->area.x);
-					wn = min(x + w ,m->area.x + m->area.width) - xn;
-					yn = max(y,m->area.y);
-					hn = min(y + h, m->area.y + m->area.height) - yn;
+		Wind_do_get(mglob.winbar,WF_FIRSTXYWH,&r.x,&r.y,&r.width,&r.height);
 	
+		while((r.width > 0) && (r.height > 0)) {
+			Objc_do_draw(global_array.int_info->vid,menu,0,9,&r);
 	
-					if((wn > 0) && (hn > 0)) {
-						objc_draw(menu,0,9,xn,yn,wn,hn);
-					};
-				};
-	
-				wind_get(m->wid,WF_NEXTXYWH,&x,&y,&w,&h);
-			};				
+			Wind_do_get(mglob.winbar,WF_NEXTXYWH,&r.x,&r.y,&r.width,&r.height);
 		};
+/*
 	}
 	else {
-		DB_printf("%s: Line %d: menu_handl_win_redraw:\r\n",
-									"Couldn't find top menu\r\n");
+
+		DB_printf("%s: Line %d: redraw_menu_bar:\r\n",
+									"Couldn't find top menu",
+									__FILE__,__LINE__);
+*/
 	};
 	
 	Rdwrs_quick(MENDREAD);
 
-	wind_update(END_UPDATE);
+	Wind_end_update();
 }
 
 /*
@@ -232,7 +205,6 @@ OBJECT *tree,     /* object tree. Pointer to the menu's OBJECT	*/
 WORD   capid)     /* Current application id. Menu owner	*/
 {
 	WORD         i;
-	REDRAWSTRUCT r;
 
   /* Lock the menu list from other processes.. 	*/
 
@@ -275,14 +247,7 @@ WORD   capid)     /* Current application id. Menu owner	*/
 		i = tree[i].ob_next;
 	};
 
-	r.type = WM_REDRAW;
-	r.sid = 0;
-	r.length = 0;
-	r.wid = mglob.winbar;
-
-	r.area = globals.screen;
-		
-	Appl_do_write(mglob.menu_handl_apid,16,&r);
+	redraw_menu_bar();
 
 	return MENU_GEN_OK;
 }
@@ -319,29 +284,10 @@ WORD                   /*                                                   */
 Menu_init_module(void) /*                                                   */
 /****************************************************************************/
 {
-	mglob.menu_handl_apid = appl_init();
-	
-	menu_register(mglob.menu_handl_apid,"  oAESis");
-	
-	recalc_tree(menutad,globals.clwidth,globals.clheight);
-	recalc_tree(informtad,globals.clwidth,globals.clheight);
-	recalc_tree(pmenutad,globals.clwidth,globals.clheight);
-
-	sprintf(versionstring,"Version %s",VERSIONTEXT);
-	((TEDINFO *)informtad[INFOVERSION].ob_spec)->te_ptext = versionstring;
-
-	menu_bar_install(menutad,mglob.menu_handl_apid);
-
-	menu_init_windows();
-
 	strcpy(progpath,"u:\\*");
 	
 	strcpy(progfile,"");
 
-	wind_set(DESKTOP_HANDLE,WF_NEWDESK,deskbgtad);
-
-	Boot_start_programs();
-							
 	return 0;
 }
 
@@ -353,8 +299,7 @@ void                   /*                                                   */
 Menu_exit_module(void) /*                                                   */
 /****************************************************************************/
 {
-  menu_shutdown();
-  appl_exit();
+  Appl_do_exit(mglob.menu_handl_apid,global_array.int_info->vid);
 }
 
 
@@ -362,35 +307,55 @@ Menu_exit_module(void) /*                                                   */
  *  Menu_handler                                                            *
  *   Handle main menu events.                                               *
  ****************************************************************************/
-WORD                   /*                                                   */
+void                   /*                                                   */
 Menu_handler(void)     /*                                                   */
 /****************************************************************************/
 {
-	WORD    kc;
-  WORD    fx,fy,fw,fh;
-	WORD    events;
-  WORD    mx,my;
+  RECT    f;
   MENUMSG	msg;
-	WORD    dummy;
+	WORD    quit = FALSE;
 
+	mglob.menu_handl_apid = Appl_do_init(&global_array);
+
+	menu_init_windows();
+
+	Menu_do_register(mglob.menu_handl_apid,"  oAESis");
+
+	menu_bar_install(globals.menutad,mglob.menu_handl_apid);
+
+
+  globals.deskbgtad[0].ob_x = globals.screen.x;
+  globals.deskbgtad[0].ob_y = globals.screen.y + globals.clheight + 3;
+  globals.deskbgtad[0].ob_width = globals.screen.width;
+  globals.deskbgtad[0].ob_height = globals.screen.height - globals.clheight - 3;
+
+	Rdwrs_operation(WSTARTWRITE);
+	Appl_set_deskbg(mglob.menu_handl_apid,global_array.int_info->vid,globals.deskbgtad);
+	Rdwrs_operation(WENDWRITE);
+
+	Boot_start_programs();
+							
   /* Start waiting for messages and rect 1 */
-  while(TRUE) {
-    events = evnt_multi(MU_MESAG | MU_KEYBD,
-    					0, 0, 0,
-    					0, 0, 0, 0, 0,
-			   			0, 0, 0, 0, 0,
-			   			(WORD *)&msg,
-			   			0,
-					  	&mx, &my, &dummy, &dummy, &kc, &dummy);
+  while(!quit) {
+  	EVENTIN  ei;
+  	EVENTOUT eo;
 
-    if(MU_MESAG & events) {
+		ei.events = MU_MESAG | MU_KEYBD;
+		  	
+		Evnt_do_multi(mglob.menu_handl_apid,
+									global_array.int_info->eventpipe,
+									global_array.int_info->msgpipe,
+									&ei,(COMMSG *)&msg,&eo,0);
+			   			
+    if(MU_MESAG & eo.events) {
       switch(msg.type) {
       case MN_SELECTED:
 				switch(msg.title) {
 				case MENU_FILE:
 					switch(msg.item) {
 						case MENU_QUIT:
-							return 0;
+							quit = TRUE;
+							break;
 
 						case MENU_STARTPROG:
 							{
@@ -400,7 +365,11 @@ Menu_handler(void)     /*                                                   */
 								WORD button;
 
 
-								fsel_exinput(progpath,progfile,&button,"Select program to start");
+								Fsel_do_exinput(mglob.menu_handl_apid,
+															global_array.int_info->vid,
+															global_array.int_info->eventpipe,
+															&button,
+															"Select program to start",progpath,progfile);
 						
 								if(button == FSEL_OK) {
 									LONG err;
@@ -427,7 +396,10 @@ Menu_handler(void)     /*                                                   */
 									Misc_setpath(oldpath);
 									
 									if(err < 0) {
-										form_error((WORD) -err - 31);
+										Form_do_error(mglob.menu_handl_apid,
+																	global_array.int_info->vid,
+																	global_array.int_info->eventpipe,
+																	(WORD) -err - 31);
 									};
 								};
 							};
@@ -438,30 +410,27 @@ Menu_handler(void)     /*                                                   */
 			  case MENU_OAESIS:
 					switch(msg.item) {
 					case MENU_INFO:
-						form_center(informtad,&fx,&fy,&fw,&fh);
-						form_dial(FMD_START,0,0,0,0,fx,fy,fw,fh);
-						objc_draw(informtad,0,9,fx,fy,fw,fh);
-						form_do(informtad,0);
-						informtad[INFOOK].ob_state &= ~SELECTED;
-						form_dial(FMD_FINISH,0,0,0,0,fx,fy,fw,fh);
+						Form_do_center(globals.informtad,&f);
+						Form_do_dial(mglob.menu_handl_apid,
+												global_array.int_info->vid,FMD_START,&f,&f);
+						Objc_do_draw(global_array.int_info->vid,globals.informtad,0,9,&f);
+						Form_do_do(mglob.menu_handl_apid,
+												global_array.int_info->vid,
+												global_array.int_info->eventpipe,
+												globals.informtad,0);
+						globals.informtad[INFOOK].ob_state &= ~SELECTED;
+						Form_do_dial(mglob.menu_handl_apid,
+												global_array.int_info->vid,FMD_FINISH,&f,&f);
 						break;
 					};
 					break;
 				};
 				break;
 	
-      /* window dirty.. 	*/
-      case WM_REDRAW:
-        handle_redraw((REDRAWSTRUCT *)&msg,0);
-        break;
-
 			case MH_TOP:
 				{
-					if(Appl_top(((COMMSG *)&msg)->msg0) != ((COMMSG *)&msg)->msg0) {
-						WORD vid = Appl_get_vid(0); /*<--*/
-						REDRAWSTRUCT rs;
-						
-						Wind_appl_top(vid,((COMMSG *)&msg)->msg0);
+					if(Appl_top(((COMMSG *)&msg)->msg0,global_array.int_info->vid) != ((COMMSG *)&msg)->msg0) {
+						Wind_appl_top(global_array.int_info->vid,((COMMSG *)&msg)->msg0);
 					
 						Rdwrs_operation(MSTARTWRITE);
 						Rdwrs_operation(ASTARTREAD);
@@ -469,13 +438,9 @@ Menu_handler(void)     /*                                                   */
 						Rdwrs_operation(AENDREAD);
 						Rdwrs_operation(MENDWRITE);
 
-						rs.type = WM_REDRAW;
-						rs.sid = 0;
-						rs.length = 0;
-						rs.wid = mglob.winbar;
-						rs.area = globals.screen;
-
-						handle_redraw(&rs,0);
+						if(Appl_get_menu(((COMMSG *)&msg)->msg0)) {
+							redraw_menu_bar();
+						};
 					};
 				};
 				break;
@@ -485,9 +450,10 @@ Menu_handler(void)     /*                                                   */
       };
     };
     
-    if(MU_KEYBD & events) {
-      if((kc & 0xff) == 'q')
-				return 0;
+    if(MU_KEYBD & eo.events) {
+      if((eo.kc & 0xff) == 'q') {
+				quit = TRUE;
+			};
     };
   };
 }
@@ -501,8 +467,6 @@ Menu_bar_remove(  /*                                                        */
 WORD apid)        /* Application whose menu is to be removed.               */
 /****************************************************************************/
 {
-	REDRAWSTRUCT	r;
-		
 	/*	Lock the menu list.. */
 	Rdwrs_operation(MSTARTWRITE);
 
@@ -510,14 +474,7 @@ WORD apid)        /* Application whose menu is to be removed.               */
 
 	Rdwrs_quick(MENDWRITE);
 
-	r.type = WM_REDRAW;
-	r.sid = 0;
-	r.length = 0;
-	r.wid = mglob.winbar;
-
-	r.area = globals.screen;
-		
-	Appl_do_write(mglob.menu_handl_apid,16,&r);
+	redraw_menu_bar();
 
 	return MENU_GEN_OK;
 }
@@ -535,11 +492,11 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
 	switch(apb->int_in[0]) {
 		case	MENU_INSTALL: 
 			apb->int_out[0] = menu_bar_install((OBJECT *)apb->addr_in[0]
-						,apb->global[2]);
+						,apb->global->apid);
 			break;
 			
 		case	MENU_REMOVE:
-			apb->int_out[0] = Menu_bar_remove(apb->global[2]);
+			apb->int_out[0] = Menu_bar_remove(apb->global->apid);
 			break;
 			
 		case	MENU_INQUIRE:
@@ -565,14 +522,10 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
 		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~CHECKED;
 		apb->int_out[0] = 1;
 		break;
-	case CHECK:
+	
+	default:
 		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= CHECKED;
 		apb->int_out[0] = 1;
-		break;
-	default:
-		DB_printf("%s: Line %d: Menu_icheck:\r\n"
-						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
-		apb->int_out[0] = 0;
 	}
 }
 
@@ -586,18 +539,14 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
 /****************************************************************************/
 {
 	switch(apb->int_in[1]) {
-	case ENABLE:
-		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~DISABLED;
-		apb->int_out[0] = 1;
-		break;
 	case DISABLE:
 		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= DISABLED;
 		apb->int_out[0] = 1;
 		break;
+
 	default:
-		DB_printf("%s: Line %d: Menu_ienable:\r\n"
-						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
-		apb->int_out[0] = 0;
+		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~DISABLED;
+		apb->int_out[0] = 1;
 	};
 }
 
@@ -615,12 +564,13 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
 		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~SELECTED;
 		apb->int_out[0] = 1;
 		break;
+
 	case HIGHLIGHT:
 		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= SELECTED;
 		apb->int_out[0] = 1;
 		break;
 	default:
-		DB_printf("%s: Line %d: Menu_ienable:\r\n"
+		DB_printf("%s: Line %d: Menu_tnormal:\r\n"
 						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
 		apb->int_out[0] = 0;
 	};
@@ -640,10 +590,10 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
 	TEDINFO *ti;
 	
 	if(ob->ob_flags & INDIRECT) {
-		ti = *(TEDINFO **)ob->ob_spec;
+		ti = ob->ob_spec.indirect->tedinfo;
 	}
 	else {
-		ti = (TEDINFO *)ob->ob_spec;
+		ti = ob->ob_spec.tedinfo;
 	};
 
 	strcpy(ti->te_ptext,
@@ -656,7 +606,7 @@ AES_PB *apb)      /* Pointer to AES parameter block.                        */
  * Menu_update_appl                                                         *
  *  Update the application menu.                                            *
  ****************************************************************************/
-void                   /*                                                   */
+WORD                   /* Top application id.                               */
 Menu_update_appl(void) /*                                                   */
 /****************************************************************************/
 {
@@ -670,52 +620,55 @@ Menu_update_appl(void) /*                                                   */
 	rwalk = PMENU_FIRST;
 	
 	while(mwalk) {
-		strcpy((BYTE *)pmenutad[rwalk].ob_spec,mwalk->ai->name);
+		strcpy(globals.pmenutad[rwalk].ob_spec.free_string,mwalk->ai->name);
 
 		if(mwalk->ai->id == topappl) {
-			pmenutad[rwalk].ob_state |= CHECKED;
+			globals.pmenutad[rwalk].ob_state |= CHECKED;
 		}
 		else {
-			pmenutad[rwalk].ob_state &= ~CHECKED;
+			globals.pmenutad[rwalk].ob_state &= ~CHECKED;
 		};
 	
-		pmenutad[rwalk].ob_flags &= ~HIDETREE;
-		pmenutad[rwalk].ob_state &= ~DISABLED;
+		globals.pmenutad[rwalk].ob_flags &= ~HIDETREE;
+		globals.pmenutad[rwalk].ob_state &= ~DISABLED;
 
 		mwalk = mwalk->mn_next;
 		rwalk++;
 	};
 	
 	if(globals.accmenu) {
-		strcpy((BYTE *)pmenutad[rwalk].ob_spec,"----------------------");
-		pmenutad[rwalk].ob_flags &= ~HIDETREE;
-		pmenutad[rwalk].ob_state &= ~CHECKED;
-		pmenutad[rwalk].ob_state |= DISABLED;
+		strcpy(globals.pmenutad[rwalk].ob_spec.free_string,
+						"----------------------");
+		globals.pmenutad[rwalk].ob_flags &= ~HIDETREE;
+		globals.pmenutad[rwalk].ob_state &= ~CHECKED;
+		globals.pmenutad[rwalk].ob_state |= DISABLED;
 		rwalk++;
 	};
 
 	mwalk = globals.accmenu;
 	
 	while(mwalk) {
-		strcpy((BYTE *)pmenutad[rwalk].ob_spec,mwalk->ai->name);
+		strcpy(globals.pmenutad[rwalk].ob_spec.free_string,mwalk->ai->name);
 
 		if(mwalk->ai->id == topappl) {
-			pmenutad[rwalk].ob_state |= CHECKED;
+			globals.pmenutad[rwalk].ob_state |= CHECKED;
 		}
 		else {
-			pmenutad[rwalk].ob_state &= ~CHECKED;
+			globals.pmenutad[rwalk].ob_state &= ~CHECKED;
 		};
 	
-		pmenutad[rwalk].ob_flags &= ~HIDETREE;
-		pmenutad[rwalk].ob_state &= ~DISABLED;
+		globals.pmenutad[rwalk].ob_flags &= ~HIDETREE;
+		globals.pmenutad[rwalk].ob_state &= ~DISABLED;
 
 		mwalk = mwalk->mn_next;
 		rwalk++;
 	};
 
-	pmenutad[rwalk].ob_flags |= HIDETREE;
+	globals.pmenutad[rwalk].ob_flags |= HIDETREE;
 	
-	pmenutad[0].ob_height = pmenutad[rwalk].ob_y;
+	globals.pmenutad[0].ob_height = globals.pmenutad[rwalk].ob_y;
+	
+	return topappl;
 }
 
 /****************************************************************************
@@ -752,8 +705,6 @@ WORD apid)        /* Application id.                                        */
 		mwalk = &(*mwalk)->mn_next;
 	};
 
-	Menu_update_appl();
-
 	Rdwrs_operation(AENDREAD);
 	Rdwrs_quick(MENDWRITE);
 }
@@ -841,6 +792,6 @@ Menu_register(    /*                                                        */
 AES_PB *apb)      /* Pointer to AES parameter block.                        */
 /****************************************************************************/
 {
-	apb->int_out[0] = Menu_do_register(/*apb->global[2],*/apb->int_in[0],
+	apb->int_out[0] = Menu_do_register(/*apb->global->apid,*/apb->int_in[0],
 													(BYTE *)apb->addr_in[0]);
 }
