@@ -45,15 +45,12 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
-#define	__TCC_COMPAT__
-
-
-#include <gemfast.h>
 #include <mintbind.h>
 #include <stdio.h>
 
 #include "appl.h"
 #include "debug.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "misc.h"
 #include "objc.h"
@@ -185,6 +182,7 @@ static void	set_write_mode(WORD vid,WORD mode) {
 		case	SWRM_TRANS		:	/*transparent*/
 			Vdi_vswr_mode(vid,MD_TRANS);
 			break;
+			
 		case	SWRM_REPLACE	:	/*replace*/
 			Vdi_vswr_mode(vid,MD_REPLACE);
 			break;
@@ -273,7 +271,7 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 	WORD tcolour = BLACK,bcolour = WHITE;
 	WORD bfill = 0;
 	WORD writemode = SWRM_TRANS;
-	WORD draw3d = 0;
+	WORD draw3d = ob->ob_flags & FLD3DANY;
 	WORD invertcolour = 0;
 	U_OB_SPEC ob_spec;
 	
@@ -318,12 +316,18 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 			
 			*pctext = '\0';
 			text = ctext;
+			
+			bfill = ob_spec.tedinfo->te_color.pattern;
+			bcolour = ob_spec.tedinfo->te_color.fillc;
 		};
 		break;
 		
 	case G_TEXT:
 	case G_BOXTEXT:
 		text = ob_spec.tedinfo->te_ptext;
+
+		bfill = ob_spec.tedinfo->te_color.pattern;
+		bcolour = ob_spec.tedinfo->te_color.fillc;
 		break;
 		
 	case G_TITLE:
@@ -336,6 +340,9 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 		ctext[0] = ob_spec.obspec.character;
 		ctext[1] = 0;
 		text = ctext;
+
+		bfill = ob_spec.obspec.fillpattern;
+		bfill = ob_spec.obspec.interiorcol;
 	};
 
 	/* set font, alignment, color and writemode */
@@ -387,9 +394,14 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 					Vdi_vst_alignment(vid,1,5,&temp,&temp);
 				}
 				
-				tcolour = ((OBJC_COLORWORD *)&ob_spec.tedinfo->te_color)->textc;
+				tcolour = ob_spec.tedinfo->te_color.textc;
 
-				writemode = ((OBJC_COLORWORD *)&ob_spec.tedinfo->te_color)->opaque;
+				if(draw3d) {
+					writemode = SWRM_TRANS;
+				}
+				else {
+					writemode = ob_spec.tedinfo->te_color.opaque;
+				};
 				break;
 				
 			case G_STRING:
@@ -420,7 +432,13 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 				Vdi_vst_alignment(vid,1,5,&temp,&temp);
 
 				tcolour = ob_spec.obspec.textcol;
-				writemode = ob_spec.obspec.textmode;
+				
+				if(draw3d) {
+					writemode = SWRM_TRANS;
+				}
+				else {
+					writemode = ob_spec.obspec.textmode;
+				};
 				break;			
 				
 			case G_BUTTON:
@@ -443,8 +461,8 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 		};
 		
 		Vdi_vst_effects(vid,txteff);
-
-		if((draw3d = ob->ob_flags & FLD3DANY)) {
+		
+		if(draw3d) {
 			if(ob->ob_state & SELECTED) {
 				if(((draw3d == FL3DIND) && ocolours.move_ind) ||
 					((draw3d == FL3DACT) && ocolours.move_act)) {
@@ -472,12 +490,30 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 					bcolour = ocolours.colour_bkg;
 					break;
 				}
+				
+				bfill = 7;
 			};
 		}
 		else {
 			if(ob->ob_state & SELECTED) {
 				invertcolour = TRUE;
 			};
+			
+			if(bfill == 0) {
+				bcolour = WHITE;
+			};
+		};
+		
+		if(!(bcolour < globals.num_pens)) {
+			bcolour = BLACK;
+		};
+		
+		if(!(tcolour < globals.num_pens)) {
+			tcolour = BLACK;
+		};
+		
+		if((tcolour == bcolour) && (bfill == 7)) {
+			tcolour = invertcolor(tcolour);
 		};
 		
 		if(invertcolour) {
@@ -489,6 +525,7 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 			set_write_mode(vid,SWRM_INVTRANS);
 			Vdi_vst_color(vid,bcolour);
 			Vdi_v_gtext(vid,tx,ty,text);
+			
 			set_write_mode(vid,SWRM_TRANS);
 		}
 		else {
@@ -501,45 +538,34 @@ static void	draw_text(WORD vid,OBJECT *ob,WORD par_x,WORD par_y,WORD is_top) {
 }
 
 static void	drawframe(WORD vid,RECT *r,WORD framesize) {
-	register WORD	incr;
-	WORD	size[10];
-	register WORD	thick;
+	WORD incr;
+	WORD size[10];
+	WORD thick;
 	
 	if(framesize) {
 		if(framesize > 0) {
 			thick = framesize;
 			incr = 1;
-			size[0] = r->x;
-			size[1] = r->y;
-			size[2] = size[0] + r->width - 1;
-			size[3] = size[1];
-			size[4] = size[2];
-			size[5] = size[1] + r->height - 1;
-			size[6] = size[0];
-			size[7] = size[5];
-			size[8] = size[0];
-			size[9] = size[1];			
+			size[0] = size[6] = size[8] = r->x;
+			size[1] = size[3] = size[9] = r->y;
+			size[2] = size[4] = size[0] + r->width - 1;
+			size[5] = size[7] = size[1] + r->height - 1;
 		}
 		else {
 			thick =  -framesize;
 			incr = -1;
-			size[0] = r->x - 1;
-			size[1] = r->y - 1;
-			size[2] = r->x + r->width;
-			size[3] = size[1];
-			size[4] = size[2];
-			size[5] = r->y + r->height;
-			size[6] = size[0];
-			size[7] = size[5];
-			size[8] = size[0];
-			size[9] = size[1];			
+			size[0] = size[6] = size[8] = r->x - 1;
+			size[1] = size[3] = size[9] = r->y - 1;
+			size[2] = size[4] = r->x + r->width;
+			size[5] = size[7] = r->y + r->height;
 		};
 			
 		while(thick > 0) {
 			Vdi_v_pline(vid,5,size);
 			thick--;
-			if(!thick)
+			if(!thick) {
 				break;
+			};
 				
 				
 			size[0] += incr;
@@ -556,10 +582,8 @@ static void	drawframe(WORD vid,RECT *r,WORD framesize) {
 	};
 }
 
-static void	drawrectfi(WORD vid,RECT *r,WORD framesize) {
-	register WORD	incr;
-	WORD	size[10];
-	register WORD	thick;
+static void	draw_filled_rect(WORD vid,RECT *r) {
+	WORD size[4];
 	
 	size[0] = r->x;
 	size[1] = r->y;
@@ -567,50 +591,6 @@ static void	drawrectfi(WORD vid,RECT *r,WORD framesize) {
 	size[3] = r->height + size[1] - 1;
 
 	Vdi_vr_recfl(vid,size);
-
-	Vdi_vsl_type(vid,SOLID);
-
-	if(framesize) {
-		if(framesize > 0) {
-			thick = framesize;
-			incr = 1;
-		}
-		else {
-			thick = -framesize;
-			incr = -1;
-			size[0]--;
-			size[1]--;
-			size[2]++;
-			size[3]++;
-		};
-		
-		size[5] = size[3];
-		size[3] = size[1];
-		size[4] = size[2];
-		size[6] = size[0];
-		size[7] = size[5];
-		size[8] = size[0];
-		size[9] = size[1];
-	
-		while(thick > 0) {
-			Vdi_v_pline(vid,5,size);
-			thick--;
-			if(!thick)
-				break;
-				
-				
-			size[0] += incr;
-			size[1] += incr;
-			size[2] -= incr;
-			size[3] += incr;
-			size[4] -= incr;
-			size[5] -= incr;
-			size[6] += incr;
-			size[7] -= incr;
-			size[8] += incr;
-			size[9] += incr;		
-		};
-	};
 }
 
 
@@ -652,8 +632,8 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 			r.width = ob->ob_width;
 			r.height = ob->ob_height;
 			
-			fillcolour = ((OBJC_COLORWORD *)&ob_spec.tedinfo->te_color)->fillc;
-			filltype = ((OBJC_COLORWORD *)&ob_spec.tedinfo->te_color)->pattern;
+			fillcolour = ob_spec.tedinfo->te_color.fillc;
+			filltype = ob_spec.tedinfo->te_color.pattern;
 		};
 		break;
 
@@ -666,7 +646,13 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 			r.height = ob->ob_height;
 			
 			fillcolour = WHITE;
-			filltype = 7;
+
+			if(ob->ob_flags & FLD3DANY) {
+				filltype = 0;
+			}
+			else {
+				filltype = 7;
+			};
 		}
 		else {
 			draw = 0;
@@ -679,12 +665,12 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 		r.width = ob->ob_width;
 		r.height = ob->ob_height;
 		
+		fillcolour = WHITE;
+
 		if(ob->ob_flags & FLD3DANY) {
-			fillcolour = WHITE;
 			filltype = 0;
 		}
 		else {
-			fillcolour = WHITE;
 			filltype = 7;
 		};
 		break;	
@@ -694,10 +680,12 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 	};
 
 	if(draw) {
-		if((mode3d = ob->ob_flags & FLD3DANY)) {
+		mode3d = ob->ob_flags & FLD3DANY;
+		
+		if(mode3d) {
 			if((mode3d == FL3DACT) || (mode3d == FL3DIND)) {
-					r.x -= 1;
-					r.y -= 1;
+					r.x--;
+					r.y--;
 					r.width += 2;
 					r.height += 2;
 			};
@@ -716,7 +704,7 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 					fillcolour = ocolours.colour_bkg;
 					break;
 				};
-				
+
 				filltype = 7;
 			};
 
@@ -733,6 +721,14 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 
 		set_write_mode(vid,SWRM_REPLACE);
 		
+		if(!(fillcolour < globals.num_pens)) {
+			fillcolour = BLACK;
+		};
+		
+		if(!(bcolour < globals.num_pens)) {
+			bcolour = BLACK;
+		};
+		
 		if(invertcolour) {
 			fillcolour = invertcolor(fillcolour);
 			bcolour = invertcolor(bcolour);
@@ -742,7 +738,7 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 			Vdi_vsf_color(vid,bcolour);
 			setfilltype(vid,7);
 	
-			drawrectfi(vid,&r,0);
+			draw_filled_rect(vid,&r);
 		};
 		
 		if(filltype != 0) {		
@@ -750,7 +746,7 @@ static void	draw_bg(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 			Vdi_vsf_color(vid,fillcolour);
 
 			setfilltype(vid,filltype);
-			drawrectfi(vid,&r,0);
+			draw_filled_rect(vid,&r);
 		};
 	};
 }
@@ -829,7 +825,18 @@ static void	draw_frame(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 			
 			framesize = ob_spec.tedinfo->te_thickness;
 
-			Vdi_vsl_color(vid,((OBJC_COLORWORD *)&ob_spec.tedinfo->te_color)->borderc);
+			Vdi_vsl_color(vid,ob_spec.tedinfo->te_color.borderc);
+		};
+		break;
+
+	case G_STRING:
+		{
+			r.x = ob->ob_x + par_x;
+			r.y = ob->ob_y + par_y;
+			r.width = ob->ob_width;
+			r.height = ob->ob_height;
+			
+			framesize = 0;
 		};
 		break;
 
@@ -920,11 +927,11 @@ static void	draw_frame(WORD vid,OBJECT *ob,WORD par_x,WORD par_y) {
 	};
 }
 
-static void drawimage(WORD vid,WORD x,WORD y,BITBLK *bb) {
+static void drawimage(WORD vid,WORD x,WORD y,BITBLK *bb,WORD state) {
 	MFDB	d,s;
 
 	WORD	xyarray[8];
-	WORD	color[2];
+	WORD	colour[2];
 	
 	xyarray[0] = 0;
 	xyarray[1] = 0;
@@ -944,21 +951,39 @@ static void drawimage(WORD vid,WORD x,WORD y,BITBLK *bb) {
 
 	(LONG)d.fd_addr = 0L;
 
-	color[0] = bb->bi_color;
+	colour[0] = bb->bi_color;
 	
-	Vdi_vrt_cpyfm(vid,MD_TRANS,xyarray,&s,&d,color);
+	if(!(colour[0] < globals.num_pens)) {
+		colour[0] = BLACK;
+	};
+	
+	if(state & SELECTED) {
+		colour[0] = invertcolor(colour[0]);
+	};
+
+	Vdi_vrt_cpyfm(vid,MD_TRANS,xyarray,&s,&d,colour);
 }
 
-static void drawicon(WORD vid,WORD x,WORD y,ICONBLK *ib) {
+static void drawicon(WORD vid,WORD x,WORD y,ICONBLK *ib,WORD state) {
 	BYTE	ch[2];
 	
 	MFDB	d,s;
 
-	WORD	xyarray[8];
-	WORD	color[2];
+	WORD xyarray[8];
+	WORD color[2];
+	WORD icon_colour = ib->ib_char >> 12;
+	WORD mask_colour = (ib->ib_char & 0x0f00) >> 8;
 	
 	RECT	r;
 	
+	if(!(mask_colour < globals.num_pens)) {
+		mask_colour = BLACK;
+	};
+	
+	if(!(icon_colour < globals.num_pens)) {
+		icon_colour = BLACK;
+	};
+	
 	xyarray[0] = 0;
 	xyarray[1] = 0;
 	xyarray[2] = ib->ib_wicon - 1;
@@ -977,12 +1002,23 @@ static void drawicon(WORD vid,WORD x,WORD y,ICONBLK *ib) {
 
 	(LONG)d.fd_addr = 0L;
 
-	color[0] = (ib->ib_char & 0x0f00) >> 8;
+	if(state & SELECTED) {
+		color[0] = icon_colour;
+	}
+	else {
+		color[0] = mask_colour;
+	};
 	
 	Vdi_vrt_cpyfm(vid,MD_TRANS,xyarray,&s,&d,color);
 
+	if(state & SELECTED) {
+		color[0] = mask_colour;
+	}
+	else {
+		color[0] = icon_colour;
+	};
+	
 	(LONG)s.fd_addr = (LONG)ib->ib_pdata;
-	color[0] = ib->ib_char >> 12;
 	Vdi_vrt_cpyfm(vid,MD_TRANS,xyarray,&s,&d,color);
 
 	r.x = x + ib->ib_xtext;
@@ -990,7 +1026,7 @@ static void drawicon(WORD vid,WORD x,WORD y,ICONBLK *ib) {
 	r.width = ib->ib_wtext;
 	r.height = ib->ib_htext;
 
-	drawtext(vid,ib->ib_ptext,&r,2,ib->ib_char >> 12,5,MD_REPLACE,0);
+	drawtext(vid,ib->ib_ptext,&r,2,ib->ib_char >> 12,5,MD_REPLACE,state);
 
 	r.x = x + ib->ib_xchar;
 	r.y = y + ib->ib_ychar;
@@ -1000,10 +1036,10 @@ static void drawicon(WORD vid,WORD x,WORD y,ICONBLK *ib) {
 	ch[0] = ib->ib_char & 0x00ff;
 	ch[1] = 0;
 
-	drawtext(vid,ch,&r,0,ib->ib_char >> 12,5,MD_TRANS,0);
+	drawtext(vid,ch,&r,0,ib->ib_char >> 12,5,MD_REPLACE,state);
 }
 
-static void drawcicon(WORD vid,WORD x,WORD y,CICONBLK *cib) {
+static void drawcicon(WORD vid,WORD x,WORD y,CICONBLK *cib,WORD state) {
 	BYTE	ch[2];
 	
 	MFDB	d,s;
@@ -1030,7 +1066,7 @@ static void drawcicon(WORD vid,WORD x,WORD y,CICONBLK *cib) {
 	};
 
 	if(!best) {
-		drawicon(vid,x,y,&cib->monoblk);
+		drawicon(vid,x,y,&cib->monoblk,state);
 	}
 	else {
 		xyarray[0] = 0;
@@ -1063,7 +1099,7 @@ static void drawcicon(WORD vid,WORD x,WORD y,CICONBLK *cib) {
 		r.width = cib->monoblk.ib_wtext;
 		r.height = cib->monoblk.ib_htext;
 	
-		drawtext(vid,cib->monoblk.ib_ptext,&r,2,cib->monoblk.ib_char >> 12,5,MD_REPLACE,0);
+		drawtext(vid,cib->monoblk.ib_ptext,&r,2,cib->monoblk.ib_char >> 12,5,MD_REPLACE,state);
 	
 		r.x = x + cib->monoblk.ib_xchar;
 		r.y = y + cib->monoblk.ib_ychar;
@@ -1073,7 +1109,7 @@ static void drawcicon(WORD vid,WORD x,WORD y,CICONBLK *cib) {
 		ch[0] = cib->monoblk.ib_char & 0x00ff;
 		ch[1] = 0;
 	
-		drawtext(vid,ch,&r,0,cib->monoblk.ib_char >> 12,5,MD_TRANS,0);
+		drawtext(vid,ch,&r,0,cib->monoblk.ib_char >> 12,5,MD_REPLACE,state);
 	};
 }
 
@@ -1099,7 +1135,7 @@ static void draw_object(WORD vid,OBJECT *tree,WORD object,RECT *clip,WORD par_x,
 	switch(type) {			
 		case G_IMAGE: /*0x17*/
 			drawimage(vid,tree[object].ob_x + par_x,tree[object].ob_y + par_y
-				,(BITBLK *)ob_spec.index);
+				,(BITBLK *)ob_spec.index,tree[object].ob_state);
 			break;
 	
 		case G_PROGDEF: /*0x18*/
@@ -1123,18 +1159,20 @@ static void draw_object(WORD vid,OBJECT *tree,WORD object,RECT *clip,WORD par_x,
 					pb.pb_parm = ob_spec.userblk->ub_parm;
 
 					remainstate = ob_spec.userblk->ub_code(&pb);
+					
+					NOT_USED(remainstate);
 				};
 			};
 			break;
 
 		case	G_ICON:
 			drawicon(vid,tree[object].ob_x + par_x,tree[object].ob_y + par_y
-				,ob_spec.iconblk);
+				,ob_spec.iconblk,tree[object].ob_state);
 			break;
 		
 		case	G_CICON:
 			drawcicon(vid,tree[object].ob_x + par_x,tree[object].ob_y + par_y
-				,(CICONBLK *)ob_spec.index);
+				,(CICONBLK *)ob_spec.index,tree[object].ob_state);
 			
 			break;
 			
@@ -1357,8 +1395,7 @@ void	do_objc_add(OBJECT *t,WORD p,WORD c) {
 
 /*objc_add	0x0028*/
 
-void	Objc_add(AES_PB *apb)
-{
+void	Objc_add(AES_PB *apb) {
 	do_objc_add((OBJECT *)apb->addr_in[0],apb->int_in[0]
 		,apb->int_in[1]);
 	
@@ -1530,17 +1567,8 @@ RECT   *clip)     /* Clipping rectangle.                                    */
 }
 
 void	Objc_draw(AES_PB *apb) {
-	WORD	vid = Appl_get_vid(apb->global[2]);
-
-	if(vid > 0) {
-		apb->int_out[0] = Objc_do_draw(vid,(OBJECT *)apb->addr_in[0],apb->int_in[0]
-			,apb->int_in[1],(RECT *)&apb->int_in[2]);
-	}
-	else {
-		apb->int_out[0] = 0;
-		DB_printf("%s: Line %d: Couldn't find application description!\r\n",
-			__FILE__,__LINE__);
-	};
+	apb->int_out[0] = Objc_do_draw(apb->global->int_info->vid,(OBJECT *)apb->addr_in[0],apb->int_in[0]
+		,apb->int_in[1],(RECT *)&apb->int_in[2]);
 }
 
 /*objc_find 0x002b*/
@@ -1718,10 +1746,8 @@ Objc_edit(        /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	WORD vid = Appl_get_vid(apb->global[2]);
-	
 	apb->int_out[1] = apb->int_in[2];
-	apb->int_out[0] = Objc_do_edit(vid,
+	apb->int_out[0] = Objc_do_edit(apb->global->int_info->vid,
 										(OBJECT *)apb->addr_in[0],apb->int_in[0],
 										apb->int_in[1],&apb->int_out[1],
 										apb->int_in[3]);
@@ -1763,9 +1789,7 @@ Objc_change(      /*                                                        */
 AES_PB *apb)      /* AES parameter block.                                   */
 /****************************************************************************/
 {
-	WORD vid = Appl_get_vid(apb->global[2]);
-	
-	apb->int_out[0] = Objc_do_change(vid,
+	apb->int_out[0] = Objc_do_change(apb->global->int_info->vid,
 										(OBJECT *)apb->addr_in[0],apb->int_in[0],
 										(RECT *)&apb->int_in[2],
 										apb->int_in[6],apb->int_in[7]);
@@ -1812,6 +1836,7 @@ WORD   *out2)     /* Outparameter 2.                                        */
 		case AD3DVAL:
 			*out1 = D3DSIZE;
 			*out2 = D3DSIZE;
+			return 1;
 		};
 	}
 	else if(mode == SV_SET) {
