@@ -24,6 +24,7 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
+#include "debug.h"
 #include "lxgemdos.h"
 #include "types.h"
 #include "vdi.h"
@@ -77,6 +78,28 @@ WORD *pxy)        /* Coordinate list.                                       */
 	vdicall(&vdipb);
 }
 
+/****************************************************************************
+ * Vdi_v_clrwk                                                              *
+ *  Clear workstation.                                                      *
+ ****************************************************************************/
+void              /*                                                        */
+Vdi_v_clrwk(      /*                                                        */
+WORD handle)      /* Workstation handle.                                    */
+/****************************************************************************/
+{
+	WORD  contrl[7];
+	VDIPB vdipb;
+	
+	contrl[0] = 3;
+	contrl[1] = 0;
+	contrl[3] = 0;
+	contrl[6] = handle;
+
+	vdipb.contrl = contrl;
+	
+	vdicall(&vdipb);
+}
+
 /****************************************************************************
  * Vdi_v_clsvwk                                                             *
  *  Close virtual workstation.                                              *
@@ -189,7 +212,7 @@ WORD *handle,     /* Workstation handle.                                    */
 WORD *work_out)   /* Out parameters.                                        */
 /****************************************************************************/
 {
-	WORD  contrl[7];
+	WORD  contrl[12];
 	VDIPB vdipb;
 	
 	contrl[0] = 100;
@@ -224,6 +247,7 @@ WORD *work_out)   /* Out parameters.                                        */
 	contrl[0] = 1;
 	contrl[1] = 0;
 	contrl[3] = 11;
+	contrl[6] = *handle;
 
 	vdipb.contrl = contrl;
 	vdipb.intin = work_in;
