@@ -13,6 +13,16 @@ WORD handle,      /* Workstation handle.                                    */
 WORD *pxy);       /* Coordinate list.                                       */
 /****************************************************************************/
 
+
+/****************************************************************************
+ * Vdi_v_clrwk                                                              *
+ *  Clear workstation.                                                      *
+ ****************************************************************************/
+void              /*                                                        */
+Vdi_v_clrwk(      /*                                                        */
+WORD handle);     /* Workstation handle.                                    */
+/****************************************************************************/
+
 /****************************************************************************
  * Vdi_v_clsvwk                                                             *
  *  Close virtual workstation.                                              *
