@@ -32,9 +32,6 @@
  * Used interfaces                                                          *
  ****************************************************************************/
 
-#define	__TCC_COMPAT__
-
-#include	<aesbind.h>
 #include	<alloc.h>
 #include	<basepage.h>
 #include	<mintbind.h>
@@ -48,9 +45,10 @@
 #include "debug.h"
 #include "evnt.h"
 #include "evnthndl.h"
+#include "gemdefs.h"
 #include "global.h"
 #include "lxgemdos.h"
-#include "memdefs.h"
+#include "mintdefs.h"
 #include "mesagdef.h"
 #include "misc.h"
 #include "objc.h"
@@ -65,9 +63,6 @@
  * Macros                                                                   *
  ****************************************************************************/
 
-#define	WIN_NR	10
-#define DRAW_SEM 0x6f445257L    /*'oDRW'*/
-
 #define W_ALL (-1)  /* set _all_ widgets */
 
 #define IMOVER 0x8000  /* Used with set_win_elem() to make icon window */
@@ -86,7 +81,7 @@ typedef struct {
  ****************************************************************************/
 
 static CSEMA update = {
-	DRAW_SEM,
+	-1,
 	-1,
 	0,
 	-1
@@ -96,7 +91,7 @@ static WORD	elemnumber = -1;
 static WORD	tednumber;
 
 
-static WINLIST *win_list[WIN_NR];
+static WINLIST *win_list = 0L;
 static WINLIST *win_free = 0L;
 
 static WORD	win_next = 0;
@@ -288,7 +283,7 @@ static OBJECT *alloc_win_elem(void) {
 	LONG    size;
 
 	while(elemnumber == -1) {
-		switch(windowtad[i].ob_type) {
+		switch(globals.windowtad[i].ob_type) {
 			case	G_TEXT		:
 			case	G_BOXTEXT	:
 			case	G_FTEXT		:
@@ -296,7 +291,7 @@ static OBJECT *alloc_win_elem(void) {
 				tnr++;
 		}
 		
-		if(windowtad[i].ob_flags & LASTOB)
+		if(globals.windowtad[i].ob_flags & LASTOB)
 		{
 			elemnumber = i + 1;
 			tednumber = tnr;
@@ -312,16 +307,16 @@ static OBJECT *alloc_win_elem(void) {
 	if(t != NULL) {
 		ti = (TEDINFO *)&t[elemnumber];
 		
-		memcpy(t,windowtad,sizeof(OBJECT) * elemnumber);
+		memcpy(t,globals.windowtad,sizeof(OBJECT) * elemnumber);
 		
 		for(i = 0; i < elemnumber; i++) {
-			switch(windowtad[i].ob_type) {
+			switch(globals.windowtad[i].ob_type) {
 				case	G_TEXT		:
 				case	G_BOXTEXT	:
 				case	G_FTEXT		:
 				case	G_FBOXTEXT	:
 					t[i].ob_spec.tedinfo = ti;
-					memcpy(ti, windowtad[i].ob_spec.tedinfo, sizeof(TEDINFO));
+					memcpy(ti, globals.windowtad[i].ob_spec.tedinfo, sizeof(TEDINFO));
 					ti++;
 			};
 		};
@@ -332,8 +327,6 @@ static OBJECT *alloc_win_elem(void) {
 
 static void set_win_elem(OBJECT *tree,WORD elem) {
 	WORD bottomsize = 0;
-	WORD headsize = 0;
-	WORD topsize = 0;
 	WORD rightsize = 0;
 	WORD left = 0,right = 0,top = 0,bottom = 0;
 
@@ -341,17 +334,6 @@ static void set_win_elem(OBJECT *tree,WORD elem) {
 		bottomsize = tree[WLEFT].ob_height + (D3DSIZE << 1);
 	};
 	
-	if((CLOSER | MOVER | FULLER | NAME | SMALLER) & elem) {
-		topsize = tree[WMOVER].ob_height + (D3DSIZE << 1);
-	};
-	
-	if(INFO & elem) {
-		headsize = topsize + tree[WINFO].ob_height + D3DSIZE * 2;
-	}
-	else {
-		headsize = topsize;
-	};
-	
 	if((LFARROW | HSLIDE | RTARROW) & elem) {
 		bottomsize = tree[WLEFT].ob_height + (D3DSIZE << 1);
 	};
@@ -571,8 +553,8 @@ static WORD	changeslider(WORD vid,WINSTRUCT *win,WORD redraw,WORD which,
 			};
 		};
 
-		newy = ((LONG)win->vslidepos * (LONG)(win->tree[WVSB].ob_height - win->tree[WVSLIDER].ob_height)) / 1000L;
-		newheight = ((LONG)win->vslidesize * (LONG)win->tree[WVSB].ob_height) / 1000L;
+		newy = (WORD)(((LONG)win->vslidepos * (LONG)(win->tree[WVSB].ob_height - win->tree[WVSLIDER].ob_height)) / 1000L);
+		newheight = (WORD)(((LONG)win->vslidesize * (LONG)win->tree[WVSB].ob_height) / 1000L);
 
 		if((win->tree[WVSLIDER].ob_y != newy) ||
 			(win->tree[WVSLIDER].ob_height != newheight)) {
@@ -610,8 +592,8 @@ static WORD	changeslider(WORD vid,WINSTRUCT *win,WORD redraw,WORD which,
 			};
 		};
 		
-		newx = ((LONG)win->hslidepos * (LONG)(win->tree[WHSB].ob_width - win->tree[WHSLIDER].ob_width)) / 1000L;
-		newwidth = ((LONG)win->hslidesize * (LONG)win->tree[WHSB].ob_width) / 1000L;
+		newx = (WORD)(((LONG)win->hslidepos * (LONG)(win->tree[WHSB].ob_width - win->tree[WHSLIDER].ob_width)) / 1000L);
+		newwidth = (WORD)(((LONG)win->hslidesize * (LONG)win->tree[WHSB].ob_width) / 1000L);
 
 		if((win->tree[WHSLIDER].ob_x != newx) ||
 			(win->tree[WHSLIDER].ob_width != newwidth)) {
@@ -720,8 +702,8 @@ static WINSTRUCT	*winalloc(void) {
 		win_next++;
 	};
 	
-	wl->next = win_list[wl->win->id % WIN_NR];
-	win_list[wl->win->id % WIN_NR] = wl;
+	wl->next = win_list;
+	win_list = wl;
 	
 	return wl->win;
 }
@@ -729,8 +711,9 @@ static WINSTRUCT	*winalloc(void) {
 /* delwinelem deletes an object created with crwinelem*/
 
 static void	delwinelem(OBJECT *o) {
-	if(o)
+	if(o) {
 		Mfree(o);
+	};
 }
 
 static WORD	top_window(WORD vid,WORD winid) { 
@@ -748,7 +731,7 @@ static WORD	top_window(WORD vid,WORD winid) {
 	WORD	dx,dy;
 	
 	if(winid == 0) {
-		if(globals.win_vis->win->status & WIN_TOPPED) {
+		if(globals.win_vis && (globals.win_vis->win->status & WIN_TOPPED)) {
 			WORD i;
 			
 			globals.win_vis->win->status &= ~WIN_TOPPED;
@@ -886,6 +869,119 @@ static WORD	top_window(WORD vid,WORD winid) {
 	return 1;
 }
 
+static WORD	bottom_window(WORD vid,WORD winid) { 
+	WORD          wastopped = 0;
+	WINSTRUCT     *newtop = 0L;
+	REDRAWSTRUCT	m;
+	
+	WINLIST	**wl = &globals.win_vis;
+	WINLIST	*ourwl;
+	
+	while(*wl) {
+		if((*wl)->win->id == winid)
+			break;
+			
+		wl = &(*wl)->next;
+	};
+		
+	if(!*wl) {
+		return 0;
+	};
+		
+	ourwl = *wl;
+		
+	*wl = (*wl)->next;
+
+	if(*wl) {
+		if((*wl)->win->status & WIN_MENU) {
+			wl = &(*wl)->next;
+		};
+	};
+
+	if((ourwl->win->status & WIN_TOPPED) && *wl) {
+		if(!((*wl)->win->status & WIN_DESKTOP)) {
+			newtop = (*wl)->win;
+			(*wl)->win->status |= WIN_TOPPED;
+			ourwl->win->status &= ~WIN_TOPPED;
+			wastopped = 1;
+		};
+	};
+
+	if(newtop) {
+		COMMSG m;
+		
+		m.type = MH_TOP;
+		m.sid = 0;
+		m.length = 0;
+		m.msg0 = newtop->owner;
+
+		Appl_do_write(0, 16, &m);
+	};
+
+	m.type = WM_REDRAW;
+	
+	if(globals.realmove) {
+		m.sid = -1;
+	}
+	else {
+		m.sid = 0;
+	};
+	
+	m.length = 0;
+
+	while(*wl) {
+		RLIST *newrects = 0L;
+		
+		if((*wl)->win->status & WIN_DESKTOP) {
+			break;
+		};
+		
+		Rlist_rectinter(&newrects,&(*wl)->win->totsize,&ourwl->win->rlist);
+		
+		Rlist_insert(&(*wl)->win->rlist,&newrects);
+
+		if(!((*wl)->win->status & WIN_MENU)) {
+			m.wid = (*wl)->win->id;
+	
+			m.area.x = ourwl->win->totsize.x;
+			m.area.y = ourwl->win->totsize.y;
+			
+			m.area.width = ourwl->win->totsize.width;
+			m.area.height = ourwl->win->totsize.height;
+		
+			if((*wl)->win != newtop) {
+				Wind_draw_elements(vid,(*wl)->win,&m.area,0);
+			};
+			
+			if(globals.realmove) {
+				m.area.x -= (*wl)->win->totsize.x;
+				m.area.y -= (*wl)->win->totsize.y;
+			};
+			
+			Appl_do_write((*wl)->win->owner,16,&m);
+		};
+		
+		wl = &(*wl)->next;
+	};
+
+	ourwl->next = *wl;
+	*wl = ourwl;
+
+	if(wastopped) {
+		WORD i;
+		
+		for(i = 0; i <= W_SMALLER; i++) {
+			set_widget_colour(ourwl->win,i,&ourwl->win->untop_colour[i],&ourwl->win->top_colour[i]);
+			set_widget_colour(newtop,i,&newtop->top_colour[i],&newtop->top_colour[i]);
+		};
+
+		Wind_draw_elements(vid,ourwl->win,&ourwl->win->totsize,0);
+		Wind_draw_elements(vid,newtop,&newtop->totsize,0);
+	};
+		
+	return 1;
+}
+
 static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {	
 	WORD dx = size->x - win->totsize.x;
 	WORD dy = size->y - win->totsize.y;
@@ -1082,7 +1178,7 @@ static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {
 						rlwalk = rlwalk->next;
 					};				
 				
-					if(rltheirnew) {
+					if(rltheirnew && !(wlwalk->win->status & WIN_DESKTOP)) {
 						m.type = WM_REDRAW;
 					
 						if(globals.realmove) {
@@ -1191,18 +1287,6 @@ static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {
 					Rlist_rectinter(&rltop,&dn,&wl->win->rlist);
 				};
 			
-				m.type = WM_REDRAW;
-				
-				if(globals.realmove) {
-					m.sid = -1;
-				}
-				else {
-					m.sid = 0;
-				};
-				
-				m.length = 0;
-				m.wid = wl->win->id;
-			
 				rl2 = rd;
 				
 				while(rl2) {
@@ -1212,14 +1296,29 @@ static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {
 					rl2 = rl2->next;
 				};
 
-				m.area = oldtotsize;
-								
+				m.type = WM_REDRAW;
+				
 				if(globals.realmove) {
-					m.area.x -= wl->win->totsize.x;
-					m.area.y -= wl->win->totsize.y;
+					m.sid = -1;
+				}
+				else {
+					m.sid = 0;
 				};
 					
-				Appl_do_write(wl->win->owner,16,&m);
+				m.length = 0;
+
+				if(rd && !(wl->win->status & WIN_DESKTOP)) {
+					m.wid = wl->win->id;
+				
+					m.area = oldtotsize;
+									
+					if(globals.realmove) {
+						m.area.x -= wl->win->totsize.x;
+						m.area.y -= wl->win->totsize.y;
+					};
+						
+					Appl_do_write(wl->win->owner,16,&m);
+				};
 
 				Rlist_insert(&wl->win->rlist,&rd);
 				
@@ -1232,9 +1331,14 @@ static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {
 
 			Rlist_rectinter(&rl2,&oldworksize,&win->rlist);
 
-			Rlist_insert(&rltop,&win->rlist);
+			Rlist_rectinter(&rltop,&win->worksize,&win->rlist);
 			
 			Rlist_insert(&win->rlist,&rl2);
+			
+			rl2 = rltop;
+			rltop = 0L;
+			
+			Rlist_insert(&rltop,&rl2);
 
 			m.wid = win->id;
 
@@ -1253,7 +1357,7 @@ static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid,WORD drawall) {
 				m.area.height = rl2->r.height;
 				
 				Appl_do_write(win->owner,16,&m);
-
+				
 				rl2 = rl2->next;
 			};
 			
@@ -1286,11 +1390,11 @@ static void	calcworksize(WORD elem,RECT *orig,RECT *new,WORD dir) {
 	WORD	topsize;
 	
 	if((HSLIDE | LFARROW | RTARROW) & elem) {
-		bottomsize = windowtad[WLEFT].ob_height + (D3DSIZE << 1);
+		bottomsize = globals.windowtad[WLEFT].ob_height + (D3DSIZE << 1);
 	};
 	
 	if((CLOSER | MOVER | FULLER | NAME) & elem) {
-		topsize = windowtad[WMOVER].ob_height + (D3DSIZE << 1);
+		topsize = globals.windowtad[WMOVER].ob_height + (D3DSIZE << 1);
 	}
 	else if(IMOVER & elem) {
 		topsize = globals.csheight + 2 + D3DSIZE * 2;
@@ -1300,7 +1404,7 @@ static void	calcworksize(WORD elem,RECT *orig,RECT *new,WORD dir) {
 	};
 	
 	if(INFO & elem) {
-		headsize = topsize + windowtad[WINFO].ob_height + 2 * D3DSIZE;
+		headsize = topsize + globals.windowtad[WINFO].ob_height + 2 * D3DSIZE;
 	}
 	else {
 		if(topsize)
@@ -1310,13 +1414,13 @@ static void	calcworksize(WORD elem,RECT *orig,RECT *new,WORD dir) {
 	};
 	
 	if((LFARROW | HSLIDE | RTARROW) & elem) {
-		bottomsize = windowtad[WLEFT].ob_height + (D3DSIZE << 1);
+		bottomsize = globals.windowtad[WLEFT].ob_height + (D3DSIZE << 1);
 	};
 	
-	if(((bottomsize < windowtad[WLEFT].ob_height) && (SIZER & elem))
+	if(((bottomsize < globals.windowtad[WLEFT].ob_height) && (SIZER & elem))
 		|| ((VSLIDE | UPARROW | DNARROW) & elem))
 	{
-		rightsize = windowtad[WSIZER].ob_width + (D3DSIZE << 1);
+		rightsize = globals.windowtad[WSIZER].ob_width + (D3DSIZE << 1);
 	};
 
 	if(dir == WC_WORK) {
@@ -1354,12 +1458,6 @@ void	Wind_init_module(void) {
   
   Vdi_v_opnvwk(work_in,&vid,work_out);
 	
-  recalc_tree(windowtad,globals.clwidth,globals.clheight);
-
-  for(i = 0; i < WIN_NR; i++) {
-    win_list[i] = 0L;
-  }
-  
   if(globals.num_pens < 16) {
   	for(i = 0; i <= W_SMALLER; i++) {
   		untop_colour[i].pattern = 0;
@@ -1371,25 +1469,18 @@ void	Wind_init_module(void) {
 	
   ws = winalloc();
   ws->status = WIN_OPEN | WIN_DESKTOP | WIN_UNTOPPABLE;
-  ws->owner = 0; 
+  ws->owner = 0;
   
-  ws->maxsize = globals.screen;
   
   ws->tree = NULL;
 
-  deskbgtad[0].ob_x = globals.screen.x;
-  deskbgtad[0].ob_y = globals.screen.y + globals.clheight + 3;
-  deskbgtad[0].ob_width = globals.screen.width;
-  deskbgtad[0].ob_height = globals.screen.height - globals.clheight - 3;
-
-  
-  ws->totsize = globals.screen;
-  
   ws->worksize.x = globals.screen.x;
   ws->worksize.y = globals.screen.y + globals.clheight + 3;
   ws->worksize.width = globals.screen.width;
   ws->worksize.height = globals.screen.height - globals.clheight - 3;
   
+  ws->maxsize = ws->totsize = ws->worksize;
+
   wl = (WINLIST *)Mxalloc(sizeof(WINLIST),GLOBALMEM);
   
   wl->win = ws;
@@ -1404,21 +1495,17 @@ void	Wind_init_module(void) {
   wl->win->rlist->r.height = globals.screen.height;
   wl->win->rlist->next = NULL;
   
-  Wind_draw_elements(vid,wl->win,&wl->win->rlist->r,0);
-
   Vdi_v_clsvwk(vid);
   
   /* Create screen drawing semaphore */
   
-  Psemaphore(0,DRAW_SEM,-1);
-  Psemaphore(3,DRAW_SEM,-1);
+	update.id = Rdwrs_create_sem();  
 }
 
 void	Wind_exit_module(void) {
   /* Destroy screen drawing semaphore */
-  
-  Psemaphore(2,DRAW_SEM,-1);
-  Psemaphore(1,DRAW_SEM,-1);
+
+	Rdwrs_destroy_sem(update.id);  
 }
 
 
@@ -1448,7 +1535,7 @@ WORD id)                /* Identification number of window.                 */
 {
 	WINLIST	*wl;
 
-	wl = win_list[id % WIN_NR];
+	wl = win_list;
 	
 	while(wl) {
 		if(wl->win->id == id)
@@ -1474,7 +1561,7 @@ WORD apid)      /* Application id.                                          */
 {
 	WORD    wid = 0;
 	WINLIST *wl;
-
+	
 	Rdwrs_operation(WSTARTREAD);
 	
 	wl = globals.win_vis;
@@ -1489,7 +1576,7 @@ WORD apid)      /* Application id.                                          */
 	};
 	
 	Rdwrs_operation(WENDREAD);
-	
+
 	top_window(vid,wid);
 	
 	return 1;
@@ -1508,31 +1595,27 @@ WORD id)        /* Identification number of window to delete.               */
 {
 	WINLIST	**wl;
 		
-	wl = &win_list[id % WIN_NR];
+	wl = &win_list;
 	
-	while(*wl)
-	{		
+	while(*wl) {		
 		if((*wl)->win->id == id)
 			break;
 	
 		wl = &(*wl)->next;
 	};
 	
-	if(!(*wl))
-	{
+	if(!(*wl)) {
 		/*no such window found! return*/
 		return 0;
 	};
 	
-	if((*wl)->win->status & WIN_OPEN)
-	{
+	if((*wl)->win->status & WIN_OPEN) {
 		Wind_do_close(vid,id);
 	};
 	
 	delwinelem((*wl)->win->tree);
 	
-	if(id == (win_next - 1))
-	{
+	if(id == (win_next - 1)) {
 		WORD	found;
 		WINLIST	*wt = *wl;
 		
@@ -1542,14 +1625,12 @@ WORD id)        /* Identification number of window to delete.               */
 		Mfree(wt);
 		win_next--;
 		
-		do
-		{
-			WINLIST 	**wlwalk = &win_list[(win_next - 1) % WIN_NR];
+		do {
+			WINLIST 	**wlwalk = &win_list;
 			
 			found = 0;
 			
-			while(*wlwalk)
-			{
+			while(*wlwalk) {
 				if((*wlwalk)->win->id == (win_next - 1))
 				{
 					found = 1;
@@ -1621,7 +1702,7 @@ WORD start)          /* Start object.                                       */
 	
 		Rdwrs_operation(ASTARTREAD);
 		
-		deskbg = Appl_get_deskbg(DESK_OWNER);
+		deskbg = Appl_get_deskbg();
 		
 		if(deskbg) {
 			while(rl) {		
@@ -1667,7 +1748,7 @@ WORD start)          /* Start object.                                       */
 	
 		Rdwrs_operation(ASTARTREAD);
 		
-		deskbg = Appl_get_deskbg(DESK_OWNER);
+		deskbg = Appl_get_deskbg();
 		
 		if(deskbg) {
 			Objc_do_draw(vid,deskbg,start,9,r);
@@ -1696,6 +1777,7 @@ WORD status)     /* Status of window.                                       */
 	WINSTRUCT	*ws;
 	
 	ws = winalloc();
+	
 	ws->status = status;
 	ws->owner = owner; 
 	
@@ -1709,8 +1791,7 @@ WORD status)     /* Status of window.                                       */
 	ws->hslidepos = 1;
 	ws->hslidesize = 1000;
 	
-	if((ws->status & WIN_DIALOG) || (ws->status & WIN_MENU))
-	{
+	if((ws->status & WIN_DIALOG) || (ws->status & WIN_MENU)) {
 		ws->tree = 0L;
 		ws->totsize = *maxsize;
 		ws->worksize = ws->totsize;
@@ -1730,6 +1811,10 @@ WORD status)     /* Status of window.                                       */
 			ws->tree[WAPP].ob_spec.tedinfo->te_ptext =
 				(char *)Mxalloc(strlen(&ai->name[2]) + 1,GLOBALMEM);
 			strcpy(ws->tree[WAPP].ob_spec.tedinfo->te_ptext,&ai->name[2]);
+			
+			if(globals.wind_appl == 0) {
+				ws->tree[WAPP].ob_spec.tedinfo->te_ptext[0] = 0;
+			};
 		};
 		
 		Rdwrs_operation(AENDREAD);
@@ -1757,7 +1842,7 @@ WORD status)     /* Status of window.                                       */
 void	Wind_create(AES_PB *apb) {	
 	Rdwrs_operation(WSTARTWRITE);
 
-	apb->int_out[0] = Wind_do_create(apb->global[2]
+	apb->int_out[0] = Wind_do_create(apb->global->apid
 		,apb->int_in[0],(RECT *)&apb->int_in[1],0);
 
 	Rdwrs_operation(WENDWRITE);
@@ -1849,7 +1934,7 @@ RECT *size)    /* Initial size of window.                                   */
 				set_widget_colour(ws,i,&ws->untop_colour[i],&ws->top_colour[i]);
 			};
 			
-			if(!(wl->win->status & WIN_DIALOG)){
+			if(!(wl->win->status & (WIN_DIALOG | WIN_MENU))){
 				m.type = WM_REDRAW;
 				m.length = 0;
 				
@@ -1873,7 +1958,7 @@ RECT *size)    /* Initial size of window.                                   */
 
 				Wind_draw_elements(vid,wl->win,&wl->win->totsize,0);				
 			
-				Appl_do_write(owner,16,&m);
+				Appl_do_write(owner,MSG_LENGTH,&m);
 			};
 
 			if(wastopped) {
@@ -1902,7 +1987,7 @@ RECT *size)    /* Initial size of window.                                   */
 void Wind_open(AES_PB *apb) {
 	Rdwrs_operation(WSTARTWRITE);
 
-	apb->int_out[0] = Wind_do_open(Appl_get_vid(apb->global[2]),
+	apb->int_out[0] = Wind_do_open(apb->global->int_info->vid,
 											apb->int_in[0],(RECT *)&apb->int_in[1]);
 
 	Rdwrs_operation(WENDWRITE);
@@ -1921,7 +2006,6 @@ WORD wid)      /* Identification number of window to close.                 */
 /****************************************************************************/
 {
 	WINLIST	  **wl = &globals.win_vis;
-	WORD      wastopped = 0;
 	WINSTRUCT *newtop = NULL;
 	
 	while(*wl) {
@@ -1935,15 +2019,13 @@ WORD wid)      /* Identification number of window to close.                 */
 		WINLIST      *wp = (*wl)->next;
 		REDRAWSTRUCT m;
 		
-		if((*wl)->win->status & WIN_TOPPED) {
+		if(((*wl)->win->status & WIN_TOPPED) && wp) {
 			(*wl)->win->status &= ~WIN_TOPPED;
 			wp->win->status |= WIN_TOPPED;
-			wastopped = 1;
 			newtop = wp->win;
 		};
 		
-		while(wp) {
-			
+		while(wp) {			
 			RLIST	*rl = NULL;
 			RLIST	*tl;
 			
@@ -1952,52 +2034,36 @@ WORD wid)      /* Identification number of window to close.                 */
 			/* Redraw "new" rectangles of windows below */
 			
 			if(rl) {
-				m.type = WM_REDRAW;
-				m.length = 0;
-				m.wid = wp->win->id;
-				
-				if(globals.realmove) {
-					m.sid = -1;
-					m.area.x = (*wl)->win->totsize.x - wp->win->totsize.x;
-					m.area.y = (*wl)->win->totsize.y - wp->win->totsize.y;
-				}
-				else {
-					m.sid = 0;
-					m.area.x = (*wl)->win->totsize.x;
-					m.area.y = (*wl)->win->totsize.y;
+				if(!(wp->win->status & (WIN_DESKTOP | WIN_MENU))) {
+					m.type = WM_REDRAW;
+					m.length = 0;
+					m.wid = wp->win->id;
+					
+					if(globals.realmove) {
+						m.sid = -1;
+						m.area.x = (*wl)->win->totsize.x - wp->win->totsize.x;
+						m.area.y = (*wl)->win->totsize.y - wp->win->totsize.y;
+					}
+					else {
+						m.sid = 0;
+						m.area.x = (*wl)->win->totsize.x;
+						m.area.y = (*wl)->win->totsize.y;
+					};
+					
+					m.area.width = (*wl)->win->totsize.width;
+					m.area.height = (*wl)->win->totsize.height;	
+	
+					Appl_do_write(wp->win->owner,16,&m);
 				};
 				
-				m.area.width = (*wl)->win->totsize.width;
-				m.area.height = (*wl)->win->totsize.height;	
-
-				Appl_do_write(wp->win->owner,16,&m);
-
 				if(wp->win != newtop) {
 					tl = rl;
-			
+				
 					while(tl) {				
-						m.type = WM_REDRAW;
-						m.length = 0;
-						m.wid = wp->win->id;
-				
-						if(globals.realmove) {
-							m.sid = -1;
-							m.area.x = tl->r.x - wp->win->totsize.x;
-							m.area.y = tl->r.y - wp->win->totsize.y;
-						}
-						else {
-							m.sid = 0;
-							m.area.x = tl->r.x;
-							m.area.y = tl->r.y;
-						};
-				
-						m.area.width = tl->r.width;
-						m.area.height = tl->r.height;	
-
 						Wind_draw_elemfast(vid,wp->win,&tl->r,0);
-
+	
 						tl = tl->next;
-					}
+					};
 				};
 						
 				Rlist_insert(&wp->win->rlist,&rl);
@@ -2014,7 +2080,7 @@ WORD wid)      /* Identification number of window to close.                 */
 		wp->win->status &= ~WIN_OPEN;
 		Mfree(wp);
 		
-		if(wastopped) {
+		if(newtop) {
 			WORD i;
 				
 			for(i = 0; i <= W_SMALLER; i++) {
@@ -2024,8 +2090,6 @@ WORD wid)      /* Identification number of window to close.                 */
 			Wind_draw_elements(vid,newtop,&newtop->totsize,0);
 		};
 
-		wp = globals.win_vis;
-		
 		return 1;
 	}
 	else
@@ -2037,7 +2101,7 @@ WORD wid)      /* Identification number of window to close.                 */
 void	Wind_close(AES_PB *apb) {
 	Rdwrs_operation(WSTARTWRITE);
 
-	apb->int_out[0] = Wind_do_close(Appl_get_vid(apb->global[2]),
+	apb->int_out[0] = Wind_do_close(apb->global->int_info->vid,
 																	apb->int_in[0]);
 
 	Rdwrs_operation(WENDWRITE);
@@ -2047,202 +2111,239 @@ void	Wind_close(AES_PB *apb) {
 /*wind_delete 0x0067*/
 void	Wind_delete(AES_PB *apb) {
 	Rdwrs_operation(WSTARTWRITE);
-	apb->int_out[0] = Wind_do_delete(Appl_get_vid(apb->global[2]),
+	apb->int_out[0] = Wind_do_delete(apb->global->int_info->vid,
 																	apb->int_in[0]);
 	Rdwrs_operation(WENDWRITE);
 }
 
 /*wind_get 0x0068*/
-void	Wind_get(AES_PB *apb) {
+
+/****************************************************************************
+ * Wind_do_get                                                              *
+ *  Implementation of wind_get().                                           *
+ ****************************************************************************/
+WORD           /* 0 if error or 1 if ok.                                    */
+Wind_do_get(   /*                                                           */
+WORD handle,   /* Identification number of window.                          */
+WORD mode,     /* Tells what to return.                                     */
+WORD *parm1,   /* Parameter 1.                                              */
+WORD *parm2,   /* Parameter 2.                                              */
+WORD *parm3,   /* Parameter 3.                                              */
+WORD *parm4)   /* Parameter 4.                                              */
+/****************************************************************************/
+{
 	WINSTRUCT	*win;
-	
+	WORD      ret;
+
 	Rdwrs_operation(WSTARTREAD);
 	
-	win = Wind_find_description(apb->int_in[0]);
+	win = Wind_find_description(handle);
 
 	if(win) {
-		switch(apb->int_in[1]) {	
+		switch(mode) {	
 			case WF_WORKXYWH	:	/*0x0004*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->worksize.x;
-				apb->int_out[2] = win->worksize.y;
-				apb->int_out[3] = win->worksize.width;
-				apb->int_out[4] = win->worksize.height;
+				ret = 1;
+				*parm1 = win->worksize.x;
+				*parm2 = win->worksize.y;
+				*parm3 = win->worksize.width;
+				*parm4 = win->worksize.height;
 				break;
 			
 			case	WF_CURRXYWH	:	/*0x0005*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->totsize.x;
-				apb->int_out[2] = win->totsize.y;
-				apb->int_out[3] = win->totsize.width;
-				apb->int_out[4] = win->totsize.height;
+				ret = 1;
+				*parm1 = win->totsize.x;
+				*parm2 = win->totsize.y;
+				*parm3 = win->totsize.width;
+				*parm4 = win->totsize.height;
 				break;
 
 			case	WF_PREVXYWH	:	/*0x0006*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->lastsize.x;
-				apb->int_out[2] = win->lastsize.y;
-				apb->int_out[3] = win->lastsize.width;
-				apb->int_out[4] = win->lastsize.height;
+				ret = 1;
+				*parm1 = win->lastsize.x;
+				*parm2 = win->lastsize.y;
+				*parm3 = win->lastsize.width;
+				*parm4 = win->lastsize.height;
 				break;
 			
 			case	WF_FULLXYWH	:	/*0x0007*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->maxsize.x;
-				apb->int_out[2] = win->maxsize.y;
-				apb->int_out[3] = win->maxsize.width;
-				apb->int_out[4] = win->maxsize.height;
+				ret = 1;
+				*parm1 = win->maxsize.x;
+				*parm2 = win->maxsize.y;
+				*parm3 = win->maxsize.width;
+				*parm4 = win->maxsize.height;
 				break;
 				
 			case WF_HSLIDE: /*0x08*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->hslidepos;
+				ret = 1;
+				*parm1 = win->hslidepos;
 				break;
 
 			case WF_VSLIDE: /*0x09*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->vslidepos;
+				ret = 1;
+				*parm1 = win->vslidepos;
 				break;
 
 			case WF_TOP: /*0x000a*/
-				apb->int_out[1] = globals.win_vis->win->id;
-				apb->int_out[2] = globals.win_vis->win->owner;
-				
-				if(globals.win_vis->next) {
-					apb->int_out[3] = globals.win_vis->next->win->id;
+				if(globals.win_vis) {
+					*parm1 = globals.win_vis->win->id;
+					*parm2 = globals.win_vis->win->owner;
+					
+					if(globals.win_vis && globals.win_vis->next) {
+						*parm3 = globals.win_vis->next->win->id;
+					}
+					else {
+						*parm3 = -1;
+					};
+					
+					ret = 1;
 				}
 				else {
-					apb->int_out[3] = -1;
+					ret = 0;
 				};
-				
-				apb->int_out[0] = 1;
 				break;
 
 			case	WF_FIRSTXYWH:	/*0x000b*/
 				win->rpos = win->rlist;
 			case	WF_NEXTXYWH:	/*0x000c*/
-				apb->int_out[0] = 1;
-				
-				while(win->rpos) {
-					if(Misc_intersect(&win->rpos->r,&win->worksize
-						,(RECT *)&apb->int_out[1]))
-						break;
+				{
+					RECT r;
 					
-					win->rpos = win->rpos->next;
-				};					
-				
-				if(!win->rpos) {
-					apb->int_out[1] = 0;
-					apb->int_out[2] = 0;
-					apb->int_out[3] = 0;
-					apb->int_out[4] = 0;
-				}
-				else	
-					win->rpos = win->rpos->next;
-				
+					ret = 1;
+					
+					while(win->rpos) {
+						if(Misc_intersect(&win->rpos->r,&win->worksize,&r)) {
+							break;
+						};
+						
+						win->rpos = win->rpos->next;
+					};					
+					
+					if(!win->rpos) {
+						*parm1 = 0;
+						*parm2 = 0;
+						*parm3 = 0;
+						*parm4 = 0;
+					}
+					else {
+						win->rpos = win->rpos->next;
+
+						*parm1 = r.x;
+						*parm2 = r.y;
+						*parm3 = r.width;
+						*parm4 = r.height;
+					};
+				};
 				break;
 
 			case WF_HSLSIZE: /*0x0f*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->hslidesize;
+				ret = 1;
+				*parm1 = win->hslidesize;
 				break;				
 
 			case WF_VSLSIZE: /*0x10*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = win->vslidesize;
+				ret = 1;
+				*parm1 = win->vslidesize;
 				break;				
 
 			case WF_SCREEN: /*0x11*/
-				apb->int_out[0] = 1;
-				apb->int_out[1] = 0;
-				apb->int_out[2] = 0;
-				apb->int_out[3] = 0;
-				apb->int_out[4] = 0;
+				ret = 1;
+				*parm1 = 0;
+				*parm2 = 0;
+				*parm3 = 0;
+				*parm4 = 0;
 				break;
 				
 			case WF_OWNER: /*0x14*/
-				{
-					WINSTRUCT *ws = Wind_find_description(apb->int_in[0]);
-				
-					if(ws) {
-						apb->int_out[0] = 1;
-						apb->int_out[1] = ws->owner;
-						if(ws->status & WIN_OPEN) {
-							WINLIST *wl = globals.win_vis;
-							
-							apb->int_out[2] = 1;
+				if(win) {
+					ret = 1;
+					*parm1 = win->owner;
+					
+					if(win->status & WIN_OPEN) {
+						WINLIST *wl = globals.win_vis;
+						
+						*parm2 = 1;
 
-							if(wl->win == ws) {
-								apb->int_out[3] = -1;
-							}
-							else {
-								while(wl->next) {
-									if(wl->next->win == ws) {
-										apb->int_out[3] = wl->win->id;
-										break;
-									};
-									
-									wl = wl->next;
+						if(wl->win == win) {
+							*parm3 = -1;
+						}
+						else if(wl) {
+							while(wl->next) {
+								if(wl->next->win == win) {
+									*parm3 = wl->win->id;
+									break;
 								};
 								
 								wl = wl->next;
 							};
 							
+							wl = wl->next;
+						};
+						
+						if(wl) {
+							wl = wl->next;
+						
 							if(wl) {
-								if((wl = wl->next)) {
-									apb->int_out[4] = wl->win->id;
-								}
-								else {
-									apb->int_out[4] = -1;
-								};
+								*parm4 = wl->win->id;
 							}
 							else {
-								apb->int_out[4] = -1;
-							};							
+								*parm4 = -1;
+							};
 						}
 						else {
-							apb->int_out[2] = 0;
-							apb->int_out[3] = -1;
-							apb->int_out[4] = -1;
-						};
+							*parm4 = -1;
+						};							
 					}
 					else {
-						apb->int_out[0] = 0;
+						*parm2 = 0;
+						*parm3 = -1;
+						*parm4 = -1;
 					};
+				}
+				else {
+					ret = 0;
 				};
 				break;
 
 			case WF_ICONIFY: /* 0x1a */
 				if(win->status & WIN_ICONIFIED) {
-					apb->int_out[1] = 1;
+					*parm1 = 1;
 				}
 				else {
-					apb->int_out[1] = 0;
+					*parm1 = 0;
 				};
 
-				apb->int_out[2] = globals.icon_width;
-				apb->int_out[3] = globals.icon_height;
+				*parm2 = globals.icon_width;
+				*parm3 = globals.icon_height;
 
-				apb->int_out[0] = 1;
+				ret = 1;
 				break;
 
 			case WF_WINX:
 			case WF_WINXCFG:
-				apb->int_out[0] = 0;
+				ret = 0;
 				break;
 
 			default:
-				DB_printf("%s: Line %d: Wind_get:\r\n"
+				DB_printf("%s: Line %d: Wind_do_get:\r\n"
 								"Unknown mode %d  (0x%x) wind_get(%d,%d,...)",
-								__FILE__,__LINE__,apb->int_in[1],apb->int_in[1],
-								apb->int_in[0],apb->int_in[1]);
-				apb->int_out[0] = 0;
+								__FILE__,__LINE__,mode,mode,
+								handle,mode);
+				ret = 0;
 		};
 	}
 	else
-		apb->int_out[0] = 0;
+		ret = 0;
 
-	Rdwrs_quick(WENDREAD);	
+	Rdwrs_quick(WENDREAD);
+	
+	return ret;
+}
+
+
+void	Wind_get(AES_PB *apb) {
+	apb->int_out[0] = Wind_do_get(apb->int_in[0],apb->int_in[1],
+																&apb->int_out[1],&apb->int_out[2],
+																&apb->int_out[3],&apb->int_out[4]);
 }
 
 
@@ -2253,11 +2354,8 @@ void	Wind_set(AES_PB *apb) {
 	Rdwrs_operation(WSTARTWRITE);
 	
 	win = Wind_find_description(apb->int_in[0]);
-	
 
 	if(win) {
-		WORD vid = Appl_get_vid(apb->global[2]);
-
 		switch(apb->int_in[1]) {	
 			case	WF_NAME	:	/*0x0002*/
 				Mfree(win->tree[WMOVER].ob_spec.tedinfo->te_ptext);
@@ -2265,7 +2363,7 @@ void	Wind_set(AES_PB *apb) {
 					(BYTE *)Mxalloc(strlen(((BYTE **)&apb->int_in[2])[0]) + 1,GLOBALMEM);
 				strcpy(win->tree[WMOVER].ob_spec.tedinfo->te_ptext,((BYTE **)&apb->int_in[2])[0]);
 				
-				Wind_draw_elements(vid,win,&win->totsize,WMOVER);
+				Wind_draw_elements(apb->global->int_info->vid,win,&win->totsize,WMOVER);
 
 				apb->int_out[0] = 1;
 				break;
@@ -2276,49 +2374,29 @@ void	Wind_set(AES_PB *apb) {
 					(BYTE *)Mxalloc(strlen(((BYTE **)&apb->int_in[2])[0]) + 1,GLOBALMEM);
 				strcpy(win->tree[WINFO].ob_spec.tedinfo->te_ptext,((BYTE **)&apb->int_in[2])[0]);
 
-				Wind_draw_elements(vid,win,&win->totsize,WINFO);
+				Wind_draw_elements(apb->global->int_info->vid,win,&win->totsize,WINFO);
 
 				apb->int_out[0] = 1;
 				break;
 				
 			case	WF_CURRXYWH	:	/*0x0005*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-					
-					if(vid > 0) {
-						apb->int_out[0] = 
-							changewinsize(win,(RECT *)&apb->int_in[2],vid,0);
-					}
-					else {
-						apb->int_out[0] = 0;
-					};
-				};
+				apb->int_out[0] = 
+					changewinsize(win,(RECT *)&apb->int_in[2],apb->global->int_info->vid,0);
 				break;
 
 			case	WF_HSLIDE: /*0x08*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-
-					apb->int_out[0] = 
-						changeslider(vid,win,1,HSLIDE,apb->int_in[2],-1);
-				};
+				apb->int_out[0] = 
+						changeslider(apb->global->int_info->vid,win,1,HSLIDE,apb->int_in[2],-1);
 				break;
 
 			case	WF_VSLIDE: /*0x09*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-
-					apb->int_out[0] = 
-						changeslider(vid,win,1,VSLIDE,apb->int_in[2],-1);
-				};
+				apb->int_out[0] = 
+					changeslider(apb->global->int_info->vid,win,1,VSLIDE,apb->int_in[2],-1);
 				break;
 
 			case	WF_TOP	:	/*0x000a*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-
-					apb->int_out[0] = top_window(vid,apb->int_in[0]);
-				};
+				apb->int_out[0] =
+						top_window(apb->global->int_info->vid,apb->int_in[0]);
 				break;
 
 			case WF_NEWDESK: /*0x000e*/
@@ -2329,7 +2407,8 @@ void	Wind_set(AES_PB *apb) {
 						globals.screen.height - (((OBJECT **)&apb->int_in[2])[0])->ob_height;
 					};
 				
-					if(Appl_set_deskbg(apb->global[2],((OBJECT **)&apb->int_in[2])[0]) == 0) {					
+					if(Appl_set_deskbg(apb->global->apid,
+							apb->global->int_info->vid,((OBJECT **)&apb->int_in[2])[0]) == 0) {
 						apb->int_out[0] = 1;
 					}
 					else {
@@ -2339,21 +2418,13 @@ void	Wind_set(AES_PB *apb) {
 				break;
 
 			case	WF_HSLSIZE: /*0x0f*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-
-					apb->int_out[0] = 
-						changeslider(vid,win,1,HSLIDE,-1,apb->int_in[2]);
-				};
+				apb->int_out[0] = 
+					changeslider(apb->global->int_info->vid,win,1,HSLIDE,-1,apb->int_in[2]);
 				break;
 
 			case	WF_VSLSIZE: /*0x10*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-
-					apb->int_out[0] = 
-						changeslider(vid,win,1,VSLIDE,-1,apb->int_in[2]);
-				};
+				apb->int_out[0] = 
+					changeslider(apb->global->int_info->vid,win,1,VSLIDE,-1,apb->int_in[2]);
 				break;
 
 			case WF_COLOR:  /*0x12*/
@@ -2378,37 +2449,24 @@ void	Wind_set(AES_PB *apb) {
 				apb->int_out[0] = 1;
 				break;
 
+			case WF_BOTTOM: /*0x0019*/
+				apb->int_out[0] =
+						bottom_window(apb->global->int_info->vid,apb->int_in[0]);
+				break;
+
 			case WF_ICONIFY: /*0x1a*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-					
-					if(vid > 0) {
-						win->origsize = win->totsize;
-						set_win_elem(win->tree,IMOVER);
-						win->status |= WIN_ICONIFIED;
-						calcworksize(IMOVER,&win->totsize,&win->worksize,WC_WORK);
-						apb->int_out[0] = changewinsize(win,(RECT *)&apb->int_in[2],vid,1);
-					}
-					else {
-						apb->int_out[0] = 0;
-					};
-				};
+				win->origsize = win->totsize;
+				set_win_elem(win->tree,IMOVER);
+				win->status |= WIN_ICONIFIED;
+				calcworksize(IMOVER,&win->totsize,&win->worksize,WC_WORK);
+				apb->int_out[0] = changewinsize(win,(RECT *)&apb->int_in[2],apb->global->int_info->vid,1);
 				break;
 
 			case WF_UNICONIFY: /*0x1b*/
-				{
-					WORD	vid = Appl_get_vid(apb->global[2]);
-					
-					if(vid > 0) {
-						set_win_elem(win->tree,win->elements);
-						win->status &= ~WIN_ICONIFIED;
-						calcworksize(win->elements,&win->totsize,&win->worksize,WC_WORK);
-						apb->int_out[0] = changewinsize(win,(RECT *)&apb->int_in[2],vid,1);
-					}
-					else {
-						apb->int_out[0] = 0;
-					};
-				};
+				set_win_elem(win->tree,win->elements);
+				win->status &= ~WIN_ICONIFIED;
+				calcworksize(win->elements,&win->totsize,&win->worksize,WC_WORK);
+				apb->int_out[0] = changewinsize(win,(RECT *)&apb->int_in[2],apb->global->int_info->vid,1);
 				break;
 
 			default:
@@ -2472,11 +2530,10 @@ void	Wind_find(AES_PB *apb) {
  *  Begin to update screen.                                                 *
  ****************************************************************************/
 WORD                   /* 1 if ok or 0.                                     */
-Wind_beg_update(       /*                                                   */
-WORD apid)             /* Application to begin update.                      */
+Wind_beg_update(void)  /*                                                   */
 /****************************************************************************/
 {
-	return Rdwrs_get_sem(apid,&update);
+	return Rdwrs_get_sem(&update);
 }
 
 /****************************************************************************
@@ -2484,40 +2541,41 @@ WORD apid)             /* Application to begin update.                      */
  *  End update of screen.                                                   *
  ****************************************************************************/
 WORD                   /* 1 if ok or 0.                                     */
-Wind_end_update(       /*                                                   */
-WORD apid)             /* Application to release exclusive right.           */
+Wind_end_update(void)  /*                                                   */
 /****************************************************************************/
 {
-	return Rdwrs_rel_sem(apid,&update);
+	return Rdwrs_rel_sem(&update);
 }
 
 /*wind_update 0x006b*/
 void	Wind_update(AES_PB *apb) {
 	switch(apb->int_in[0]) {
 	case	BEG_UPDATE + 0x100: /* check & set */
-		if ((update.count > 0) && (apb->global[2] != update.apid)) {
+		if((update.count > 0) && (apb->global->apid != update.apid)) {
 			apb->int_out[0] = 0;
 			break;
 		}
 
 	case	BEG_UPDATE:
-		apb->int_out[0] = Wind_beg_update(apb->global[2]);
+		apb->int_out[0] = Wind_beg_update();
 		break;
 
 	case	END_UPDATE:
-		apb->int_out[0] = Wind_end_update(apb->global[2]);
+		apb->int_out[0] = Wind_end_update();
 		break;
 
-	case	BEG_MCTRL + 0x100: /* check & set */
-		apb->int_out[0] = Evhd_gain_mctrl(apb->global[2],MCTRL_BUTNB);
+/*	
+	case	BEG_MCTRL + 0x100:
+		apb->int_out[0] = Evhd_gain_mctrl(apb->global->apid);
 		break;
+*/
 
 	case BEG_MCTRL:
-		apb->int_out[0] = Evhd_gain_mctrl(apb->global[2],MCTRL_BUTTON);
+		apb->int_out[0] = Evhd_gain_mctrl(apb->global->apid,FALSE);
 		break;
 
 	case END_MCTRL:
-		apb->int_out[0] = Evhd_release_mctrl(apb->global[2]);
+		apb->int_out[0] = Evhd_release_mctrl();
 		break;
 
 	default:
@@ -2536,3 +2594,60 @@ void	Wind_calc(AES_PB *apb) {
 	
 	apb->int_out[0] = 1;
 }
+
+/****************************************************************************
+ * Wind_do_new                                                              *
+ *  Implementation of wind_new().                                           *
+ ****************************************************************************/
+WORD           /* 0 if error or 1 if ok.                                    */
+Wind_do_new(   /*                                                           */
+WORD vid,      /* VDI workstation id.                                       */
+WORD apid)     /* Application whose windows should be erased.               */
+/****************************************************************************/
+{
+	WINLIST *wl;
+	
+	Rdwrs_operation(WSTARTWRITE);
+	
+	wl = globals.win_vis;
+
+	while(wl) {
+		if((wl->win->owner == apid) && !(wl->win->status & WIN_DESKTOP)) {
+			Wind_do_delete(vid,wl->win->id);
+			
+			wl = globals.win_vis;
+		}
+		else {
+			wl = wl->next;
+		};
+	};
+	
+	wl = win_list;
+	
+	while(wl) {
+		if(wl->win->owner == apid) {
+			Wind_do_delete(vid,wl->win->id);
+			
+			wl = win_list;
+		}
+		else {	
+			wl = wl->next;
+		};
+	};
+
+	Rdwrs_operation(WENDWRITE);
+	
+	return 1;
+}
+
+/****************************************************************************
+ * Wind_new                                                                 *
+ *  0x006d wind_new().                                                      *
+ ****************************************************************************/
+void              /*                                                        */
+Wind_new(         /*                                                        */
+AES_PB *apb)      /* AES parameter block.                                   */
+/****************************************************************************/
+{
+	apb->int_out[0] = Wind_do_new(apb->global->int_info->vid,apb->global->apid);
+}
\ No newline at end of file
